import os
import time
import getpass
from laipvt.sysutil.gvalue import CHECK_FILE, LOG_TO_TTY
from laipvt.sysutil.ssh import SSHConnect
from laipvt.sysutil.log import Logger
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.util import path_join, log, get_yaml_config


class RecoverMysql:
	""" 恢复mysql """

	def __init__(self):
		self.check_result_file = CHECK_FILE
		self.check_result = CheckResultHandler(self.check_result_file)
		self.deploy_dir = self.check_result.deploy_dir
		self.mysql_dir = path_join(self.deploy_dir, "mysql")
		self.container_name = get_yaml_config(path_join(self.mysql_dir, "docker-compose.yml"))["services"]["mysqlmgr"][
			"container_name"]
		self.log = Logger(
			tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY)
		).get()
		self.master_ip = input("请输入当前Mgr集群主节点地址: ")
		self.slive_ip = input("请输入当前Mgr集群要恢复的节点地址: ")
		if self.master_ip == self.slive_ip:
			log.error("主节点与恢复节点地址不应一致")
			exit(2)
		self.mysql_pass = input("请输入mysql密码: ")
		self.mysql_port = input("请输入mysql端口: ")
		self.service_username = input("请输入登陆入服务器用户: ")
		self.service_password = getpass.getpass("请输入登陆入服务器密码(已隐藏): ********")
		self.service_ssh = input("请输入SSH端口号: ")
		self.slive_client = f"mysql -uroot -p{self.mysql_pass}  -h{self.slive_ip} -P{self.mysql_port}"
		self.master_client = f"mysql -uroot -p{self.mysql_pass}  -h{self.master_ip} -P{self.mysql_port}"
		self.find_master = "'select MEMBER_HOST, MEMBER_PORT,MEMBER_STATE from " \
		                   "performance_schema.replication_group_members m inner " \
		                   "join performance_schema.global_status g on m.MEMBER_ID=g.VARIABLE_VALUE  and VARIABLE_NAME = " \
		                   "\"group_replication_primary_member\";'"
		self.start_mgr = f"'CHANGE MASTER TO MASTER_USER=\"rpl_user\", MASTER_PASSWORD=\"{self.mysql_pass}\" FOR CHANNEL " \
		                 "\"group_replication_recovery\"; set global group_replication_allow_local_disjoint_gtids_join=on;" \
		                 "start group_replication;'"
		self.count_mgr = "'select count(*) from performance_schema.replication_group_members where MEMBER_STATE = " \
		                 "\"ONLINE\";' "

		self.reset_slave = "'reset master;set sql_log_bin=0;flush privileges;set sql_log_bin=1;'"

		self.count_proxsql = "mysql -uadmin -padmin --prompt='proxysql> ' -P6032 -h127.0.0.1 -e 'select * from " \
		                     "runtime_mysql_servers';"

	def check_disk(self):
		several = self.master_ip, self.slive_ip
		for ip in several:
			cmd1 = f"du -sm {self.mysql_dir} |awk '{{print $1}}'"
			data_size = self.sshclient(cmd1, ip)["stdout"].replace("\n", "").replace("\r", "")
			cmd2 = f"df -m {self.mysql_dir}|tail -n 1 |awk '{{print $4}}'"
			disk_size = self.sshclient(cmd2, ip)["stdout"].replace("\n", "").replace("\r", "")
			if int(float(data_size)) * 3 < int(float(disk_size)):
				log.info(f"{ip}节点目录{self.mysql_dir}共占用{data_size}MB，数据盘共剩余{disk_size}MB")
			else:
				log.info(f"{ip}节点目录{self.mysql_dir}共占用{data_size}MB, 数据盘共剩余{disk_size}MB,磁盘空间不够，请清理磁盘空间...")
				exit(2)

	def rename_dir(self, dir_path):
		cmd = f"mv -f {dir_path} {path_join(self.mysql_dir, 'mysql_data_bak')}"
		return cmd

	def backup_data(self):
		cmd = f"docker exec -i {self.container_name} mysqldump  --set-gtid-purged=ON --single-transaction   --all-databases " \
		      f"-u{'root'} -p{self.mysql_pass} -h {self.master_ip} -P{self.mysql_port} > {self.mysql_dir}/recover.sql"
		cmd2 = f"docker exec -i {self.container_name} mysqldump --databases --routines  sys  --set-gtid-purged=off -u{'root'}" \
		       f" -p{self.mysql_pass} -h {self.master_ip} -P{self.mysql_port} > {self.mysql_dir}/recover_function.sql"
		res = self.sshclient(cmd, self.master_ip)
		res1 = self.sshclient(cmd2, self.master_ip)
		if res["code"] == 0 and res1["code"] == 0:
			log.info("数据备份成功...")
		else:
			log.error(f"备份命令：{cmd}")
			log.error("数据备份失败，请检查")
			log.error(res["stdout"])
			exit(2)

	def ask_master(self):
		cmd = f"{self.master_client} -e {self.find_master}"
		res = self.sshclient(cmd, self.master_ip)
		log.info(res)
		for res1 in res["stdout"].split('\n'):
			if any(digital.isdigit() for digital in res1):
			# if '.' in res1:
				res1 = res1.split()
				log.info(res1)
				if res1[0] == self.master_ip and res1[1] == self.mysql_port and res1[2] == "ONLINE":
					log.info(f"Master主节点正常: {res1}...")
				else:
					log.error("Master节点异常或master主节点地址填写错误")
					exit(2)
			else:
				pass

	def rename_mysql_dir(self):
		# try:
		# self.sshclient(shutil.move(path_join(self.mysql_dir, "data"), path_join(self.mysql_dir, "data_bak")), self.slive_ip)
		local_path = path_join(self.mysql_dir, "data")
		# srcpath = path_join(self.mysql_dir, "data_bak")
		# cmd = "mv {local_path} {srcpath}".format(local_path=local_path, srcpath=srcpath)
		# log.info(cmd)
		res = self.sshclient(self.rename_dir(local_path), self.slive_ip)
		if res["code"] != 0:
			log.error(res["stdout"])
			log.error("数据目录移动失败，请检查...")
			exit(2)

	# except Exception as e:
	# 	log.error(e)
	# 	exit(2)

	def remove_mysql(self):
		cmd = f"docker rm -f {self.container_name}"
		res = self.sshclient(cmd, self.slive_ip)
		if res["code"] == 0:
			log.info(f"{self.slive_ip}节点{self.container_name}容器已删除...")
		else:
			log.error(res["stdout"])
			exit(2)

	def start_mysql(self):
		mysql_compose = f"{self.deploy_dir}/{'mysql'}/{'docker-compose.yml'}"
		cmd = f"docker-compose -f {mysql_compose} up -d "
		self.sshclient(cmd, self.slive_ip)
		log.info(f"{self.slive_ip}节点mysql容器启动成功，等待60s...")
		time.sleep(60)

	def pre_mysql(self):
		# cmd = f"{self.slive_client} -e 'reset master;set sql_log_bin=0;flush privileges;set sql_log_bin=1;'"
		cmd = f"{self.slive_client} -e {self.reset_slave}"
		self.sshclient(cmd, self.slive_ip)

	def import_data(self):
		cmd = f"{self.slive_client}  < {self.mysql_dir}/recover.sql"
		res = self.sshclient(cmd, self.master_ip)
		if res["code"] == 0:
			log.info("备份数据导入成功...")
		else:
			log.error("备份数据导入失败，请检查")
			log.error(res["stdout"])
			exit(2)

	def import_funtion(self):
		cmd = f"{self.slive_client}   < {self.mysql_dir}/recover_function.sql"
		res = self.sshclient(cmd, self.master_ip)
		if res["code"] != 0:
			log.error("函数导入失败，请检查")
			log.error(res["stdout"])
			exit(2)

	def start_slavemgr(self):
		cmd = f"{self.slive_client} -e {self.start_mgr}"
		res = self.sshclient(cmd, self.slive_ip)
		if res["code"] == 0:
			log.info(f"{self.slive_ip} 节点Mgr启动成功...")
		else:
			log.error(f"{self.slive_ip} 节点Mgr启动失败,请检查")
			log.error(res["stdout"])
			exit(2)

	def check_mgrmembers(self):
		cmd = f"{self.slive_client} -e {self.count_mgr}"
		res = self.sshclient(cmd, self.slive_ip)
		log.info(res["stdout"].split())
		if res["stdout"].split()[-1] == "3":
			log.info("Mysql mgr集群正常,ONLINE节点数为3...")
			log.info(f"请登陆mysql中手动检查确认ONLINE数为3,命令: {cmd}")
			log.info(f"请登陆proxysql中手动确认集群正常,命令: {self.count_proxsql}")
			log.info(
				f"\033[1;35m 如确认集群正常确认无问题后！！！，请将节点 {self.slive_ip}  {path_join(self.mysql_dir, 'data_bak')} "
				f"备份文件清理，避免占用磁盘空间\033[0m!")
		else:
			log.error(f"Mysql mgr集群异常，请检查{res['stderr']}")
			exit(2)

	def flush(self):
		cmd = f"{self.slive_client} -e 'flush privileges;'"
		self.sshclient(cmd, self.slive_ip)

	def sshclient(self, cmd, ip):
		ssh_cli = SSHConnect(hostip=ip, username=self.service_username, password=self.service_password,
		                     port=self.service_ssh)
		self.log.info(f"{ip} 执行 {cmd} 命令")
		res = ssh_cli.run_cmd(cmd)
		ssh_cli.close()
		return res


if __name__ == '__main__':
	mysql = RecoverMysql()
	mysql.check_disk()
	mysql.ask_master()
	mysql.backup_data()
	mysql.rename_mysql_dir()
	mysql.remove_mysql()
	mysql.start_mysql()
	mysql.pre_mysql()
	mysql.import_data()
	mysql.import_funtion()
	mysql.start_slavemgr()
	mysql.flush()
	mysql.check_mgrmembers()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from laipvt.sysutil.args import Args

def main():
    args = Args().parse_args()
    func = args.which
    if func == "delete":
        from laipvt.cli.delete import delete_main
        delete_main(args)
    else:
        from laipvt.cli.deploy import deploy_main
        deploy_main(args)
    




from __future__ import absolute_import
from __future__ import unicode_literals

import docker

from kubernetes import client, config
from kubernetes.config.kube_config import KUBE_CONFIG_DEFAULT_LOCATION
from laipvt.helper.exception import ModelError
from laipvt.sysutil.util import log, path_join
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.helper.errors import Helper

class DockerModel():
    def __init__(self, url=None):
        self.client = docker.APIClient(base_url=url, timeout=300)

    def _image_generator_parse(self, gen: any) -> str:
        image_name = next(gen)["stream"].split(" ")[-1].replace("\n", "")
        return image_name

    def _parse_tag(self, image: str) -> dict:
        """
        :param image: tfserver:2.2.0-gpu
        :return: {"image_name": tfserver, "image_tag": 2.2.0-gpu}
        """
        try:
            t = image.split(":")
            return {"image_name": t[0], "image_tag": t[1]}
        except AttributeError as e:
            log.error("镜像{}导入失败：{}".format(image, e))
        return {"image_name": "", "image_tag": "t[1]"}

    def load(self, image: str) -> str:
        with open(image, "rb") as f:
            try:
                r = self.client.load_image(f, quiet=True)
            except Exception as e:
                log.error(e)
        try:
            return self._image_generator_parse(r)
        except Exception as e:
            log.error("导入镜像{}失败".format(image))
            log.error(e)

    def tag(self, old_image_withtag: str, new_image: str, new_image_tag: str, force=True) -> bool:
        try:
            return self.client.tag(old_image_withtag, new_image, new_image_tag, force=force)
        except Exception as e:
            log.error(e)

    def push(self, image: str, tag: str, auth_config=None) -> bool:
        res = list(self.client.push(image, tag, stream=True, decode=True, auth_config=auth_config))[-1]
        if "progressDetail" in res:
            return True
        if "errorDetail" in res:
            raise ModelError("DockerModel.push %s" % res.get("error")) from None
        raise ModelError("DockerModel.push %s" % res) from None


class DockerImageModel(DockerModel):
    def __init__(self, image: str, project: str, repo: str, tag_version: str):
        """
        :param image: /tmp/etcd.tar
        :param project: middleware
        :param repo: 1.1.1.1:8080
        docker load < /tmp/etcd.tar
        docker tag registry.cn-beijing.aliyuncs.com/laiye_poc/etcd:latest 1.1.1.1:8080/middleware/etcd:latest
        docker push 1.1.1.1:8080/middleware/etcd:latest
        """
        super(DockerImageModel, self).__init__()
        self.image = image # 要导入的镜像文件路径
        self.repo_path = "{}/{}".format(repo, project)
        self.new_image_addr = None
        self.tag_version = tag_version
        try:
            harbor_cfg = HarborConfigHandler().get_config_with_check_result()
            self.auth_config = {
                "username": "admin",
                "password": harbor_cfg["harbor"]["password"]
            }
        except Exception as e:
            log.error(e)
            self.auth_config = None

    def _parse_tag(self, image: str) -> dict:
        """
        :param image: tfserver:2.2.0-gpu
        :return: {"image_name": tfserver, "image_tag": 2.2.0-gpu}
        """
        try:
            t = image.split(":")
            return {"image_name": t[0], "image_tag": self.tag_version}
        except AttributeError as e:
            log.error(Helper().PUSH_IMAGE_ERROR.format(image, e))
            exit(2)
        return {"image_name": "", "image_tag": self.tag_version}

    def run(self) -> bool:
        image_name_withtag = self.load(self.image)
        image_name = self._parse_tag(image_name_withtag)
        # t = image_name_withtag.split(":")
        # image_name = {"image_name": t[0], "image_tag": self.tag_version}
        new_image_name = image_name["image_name"].split("/")[-1]
        log.info(Helper().TAG_IMAGE.format(
            image_name_withtag,
            path_join(self.repo_path, new_image_name),
            image_name["image_tag"]
        ))
        if self.tag(image_name_withtag, path_join(self.repo_path, new_image_name), image_name["image_tag"]):
            self.new_image_addr = path_join(self.repo_path, new_image_name)
            return self.push(path_join(self.repo_path, new_image_name), image_name["image_tag"], auth_config=self.auth_config)
        return False

class ComposeModel(object):
    """
    :return string
    """
    def __init__(self, compose_file: str, compose_cmd="/usr/bin/docker-compose"):
        """

        :param compose_file: string docker-compose路径
        """
        # try:
        #     if get_yaml_config(compose_file):
        self.compose_file = compose_file
        #     else:
        #         raise ModelError("ComposeModel.init %s 不是一个合法的docker-compose文件" % compose_file) from None
        # except UtilsError:
        #     raise ModelError("ComposeModel.init %s 不是一个合法的docker-compose文件" % compose_file) from None

        # try:
        #     if file_run_able(compose_cmd):
        self.compose = compose_cmd
        #     else:
        #         raise ModelError("ComposeModel.init %s 不是一个可执行文件" % compose_cmd)
        # except UtilsError as e:
        #     raise ModelError("ComposeModel.init %s" % e.msg) from None

    def up(self) -> str:
        return "%s -f %s up -d" % (self.compose, self.compose_file)

    def down(self) -> str:
        return "%s -f %s down" % (self.compose, self.compose_file)

    def restart(self) -> str:
        return "%s -f %s restart" % (self.compose, self.compose_file)

class KubeModel():
    def __init__(self, config_file=KUBE_CONFIG_DEFAULT_LOCATION):
        config.load_kube_config(config_file=config_file)
        self.cli = client.CoreV1Api()

    def create_namespace(self, namespace: str) -> dict:
        """

        :param namespace: string namespace名称
        :return: {name: namespace, status: Active}
        """
        body = client.V1Namespace()
        body.metadata = client.V1ObjectMeta(name=namespace)
        try:
            info = {}
            res = self.cli.create_namespace(body)
            info["name"] = namespace
            info["status"] = res.status.phase
            return info
        except client.exceptions.ApiException as apierror:
            raise ModelError("KubeModel.create_namespce API错误: %s" % apierror.reason) from None

    def get_node_status(self) -> list:
        """

        :return: [{node_name: "name", node_status: "Ready"}]
        """
        all_nodes = self.cli.list_node(pretty='true')
        node_info = []
        for node in all_nodes.items:
            info = {}
            info['name'] = node.metadata.name
            for st in node.status.conditions:
                if st.reason == "KubeletReady":
                    info["status"] = st.type
            node_info.append(info)
        return node_info

    def get_all_pod_status(self, namespace="all_namespaces") -> list:
        """
        获取所有命令空间下的pod状态
        : return list

        [{
            "pod_ip":"10.244.0.19",
            "namespace":"mage",
            "pod_name":"ocr-text-recognition-tf-server-76d9c59d97-nt7lh",
            "status":"Running"
        },
        {
            "pod_ip":"10.244.0.25",
            "namespace":"mid",
            "pod_name":"license-manager-7d56c5f687-nsmzx",
            "status":"Running"
        }]
        """
        if namespace == "all_namespaces":
            ret = self.cli.list_pod_for_all_namespaces(watch=False, pretty='true')
        else:
            ret = self.cli.list_namespaced_pod(namespace=namespace)

        pod_info = []
        for item in ret.items:
            pod_info.append(
                {
                    "pod_ip": item.status.pod_ip,
                    "namespace": item.metadata.namespace,
                    "pod_name": item.metadata.name,
                    "status": item.status.phase
                }
            )
        return pod_info



from __future__ import absolute_import
from __future__ import unicode_literals

import json
import requests
from requests.adapters import HTTPAdapter
from requests.auth import HTTPBasicAuth
from laipvt.helper.errors import Helper
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.util import path_join, ssh_obj, log, status_me, post, get, put
from laipvt.sysutil.template import FileTemplate
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.command import BASH_RUN, TARGZ_DECOMPRESSION, CHMOD_USER_GROUP, ENABLE_HARBOR_SERVICE


class HarborController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: HarborConfigHandler, template: str):
        super(HarborController, self).__init__(result, handler, template)

        self.harbor_conf_tmp = path_join("/tmp", "harbor.yml")
        self.harbor_conf_template = path_join(self.template, "config.tmpl")
        self.harbor_conf_file = path_join(self.base_dir, "harbor.yml")
        self.harbor_service_template = path_join(self.template, "harbor.service.tmpl")
        self.harbor_service_tmp = path_join("/tmp", "harbor.service")
        self.harbor_service_file = "/usr/lib/systemd/system/harbor.service"
        self.habor_data = path_join(self.template, "data")
        self._docker_compose_file = path_join(self.base_dir, "docker-compose.yaml")
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        self.harbor_cfg["harbor"]["log_location"] = path_join(self.base_dir, "var/log/harbor")
        self.harbor_cfg["harbor"]["data_volume"] = path_join(self.base_dir, "data")
        self.harbor_cfg["harbor"]["deploy_path"] = self.base_dir
        self.harbor_cfg["harbor"]["ipaddress"] = [ip.ipaddress for ip in self.harbor_server]
        self.harbor_nginx_tmp = path_join("/tmp", "nginx-harbor.conf")
        self.harbor_nginx_template = path_join(self.template, "nginx-harbor.tmpl")
        self.harbor_nginx_file = path_join(self.deploy_dir, "nginx/http/nginx-harbor.conf")
        self.master_harbor = self.harbor_server[0]

    def _generic_config(self):
        """
        如果是多机Harbor 只将data目录拷贝到第一台harbor上，传输第二台的时候将data.tar.gz过滤掉，使用内部同步机制，节约部署时间
        :return:
        """
        for num_id, server in enumerate(self.harbor_server):
            self.harbor_cfg["harbor"]["localhost_ip"] = server.ipaddress
            FileTemplate(self.harbor_cfg, self.harbor_conf_template, self.harbor_conf_tmp).fill()
            FileTemplate(self.harbor_cfg, self.harbor_service_template, self.harbor_service_tmp).fill()
            if num_id == 0:
                self.send_config_file(server=server, src=self.template, dest=self.base_dir)
            else:
                self.send_config_file(server=server, src=self.template, dest=self.base_dir, ignore='data.tar.gz')
            self.send_config_file(server=server, src=self.harbor_conf_tmp, dest=self.harbor_conf_file)
            self.send_config_file(server=server, src=self.harbor_service_tmp, dest=self.harbor_service_file)

    def ssh_cli(self, cmd):
        for server in self.harbor_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(cmd)
            ssh_cli.close()
        return res

    def _install_harbor(self):
        cmd = BASH_RUN.format(path_join(self.base_dir, "install.sh"))
        res = self.ssh_cli(cmd)
        if res["code"] != 0:
            log.error("{} {}".format(res["stdout"], res["stderr"]))
            exit(2)

    def _unpack_data(self):
        """
        只解压第一台Harbor的数据目录
        :return:
        """
        ssh_cli = ssh_obj(
            ip=self.master_harbor.ipaddress,
            user=self.master_harbor.username,
            password=self.master_harbor.password,
            port=self.master_harbor.port
        )
        cmd = [
            TARGZ_DECOMPRESSION.format(path_join(self.base_dir, "data.tar.gz"), self.base_dir),
            CHMOD_USER_GROUP.format(
                "999", "999",
                path_join(self.base_dir, "data", "database"),
                path_join(self.base_dir, "data", "redis")),
            CHMOD_USER_GROUP.format(
                "10000", "10000",
                path_join(self.base_dir, "data", "secret"),
                path_join(self.base_dir, "data", "registry"))
        ]
        res_list = ssh_cli.run_cmdlist(cmd)
        for res in res_list:
            if res["code"] != 0:
                log.error(Helper().DECOMPRESS_HARBOR_DATA_ERROR.format(res["stdout"], res["stderr"]))
                exit(2)

    def _start(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.harbor_server:
            log.info(Helper().START_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
                res = ssh_cli.run_cmd(ENABLE_HARBOR_SERVICE)
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _check(self):
        super().wait_for_service_start()
        s = requests.Session()
        s.mount('http://', HTTPAdapter(max_retries=3))
        s.mount('https://', HTTPAdapter(max_retries=3))
        try:
            for server in self.harbor_server:
                url = "http://{IP}:{PORT}/api/v2.0/health".format(
                    IP=server.ipaddress, PORT=self.harbor_cfg["harbor"]["http_port"]
                )
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE.format(self.middleware_name,
                                                                  server.ipaddress,
                                                                  self.harbor_cfg["harbor"]["http_port"]))
                result = s.get(url, timeout=5).json()
                if result["status"] == "healthy":
                    log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED.format(server.ipaddress,
                                                                                   self.harbor_cfg["harbor"]["http_port"]))
                else:
                    log.error(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(server.ipaddress,
                                                                                   self.harbor_cfg["harbor"]["http_port"]))
                    log.error(result)
                    exit(2)
        except Exception as e:
            log.error(e)
            exit(2)

    def _change_password(self):
        """
        只修改第一台Harbor的密码即可，第二台使用Habor配置文件中的密码，所以无需修改
        :return:
        """
        log.info(Helper().CHANGE_HARBOR_PASSWORD)
        try:
            url = "http://{IP}:{PORT}/api/v2.0/users/1/password".format(
                IP=self.master_harbor.ipaddress, PORT=self.harbor_cfg["harbor"]["http_port"])
            headers = {"Content-Type": "application/json"}
            data_dit = {"old_password": "Harbor12345", "new_password": "{}".format(self.harbor_cfg["harbor"]["password"])}
            result = requests.put(url=url, json=data_dit, headers=headers, auth=HTTPBasicAuth('admin', 'Harbor12345'))
            if result:
                log.info(Helper().CHANGE_HARBOR_PASSWORD_SUCCEED)
            else:
                log.error(Helper().CHANGE_HARBOR_PASSWORD_FAILED)
                log.error(result)
                exit(2)
        except Exception as e:
            log.error(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(self.master_harbor.ipaddress,
                                                                           self.harbor_cfg["harbor"]["http_port"]))
            log.error(e)
            exit(2)

    def registries(self):
        """
        两台Harbor互相配置仓库管理
        :return:
        """
        for num_id, ip in enumerate(self.harbor_server):
            if num_id == 0:
                harbor_ipaddress = self.harbor_server[1].ipaddress
                # url = "http://{}:{}/api/v2.0/registries".format(self.harbor_server[1].ipaddress, self.harbor_cfg["harbor"]["harbor_http_port"])
            else:
                harbor_ipaddress = self.harbor_server[0].ipaddress
                # url = "http://{}:{}/api/v2.0/registries".format(self.harbor_server[0].ipaddress, self.harbor_cfg["harbor"]["harbor_http_port"])
            url = "http://{}:{}/api/v2.0/registries".format(harbor_ipaddress,
                                                            self.harbor_cfg["harbor"]["harbor_http_port"])
            data = {
                "credential": {
                    "access_key": "admin",
                    "access_secret": "{}".format(self.harbor_cfg["harbor"]["password"]),
                    "type": "basic"
                },
                "name": "Harbor_slave",
                "url": "http://{}:{}".format(self.harbor_server[num_id].ipaddress,
                                             self.harbor_cfg["harbor"]["harbor_http_port"]),
                "insecure": False,
                "id": 100,
                "type": "harbor"
            }
            res = self.interface_res(url, data)
            if res.status_code != 201:
                log.error(Helper().HARBOR_REG_CONFIG_FAILED.format(harbor_ipaddress, res.status_code))
                exit(2)

    def interface_res(self, url, data):
        try:
            res = post(url, data=json.dumps(data), timeout=5,
                       auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            return res
        except Exception as e:
            log.error(e)
            exit(2)

    def policies(self, model, policy_id=''):
        """
        配置Harbor复制管理，首次调用需要设置手动同步模式，后面手动调用手动出发同步接口，手动同步完毕后，在将模式改为触发同步
        :return:
        """
        for num_id, server in enumerate(self.harbor_server):
            if num_id == 0:
                harbor_ipaddress = self.harbor_server[1].ipaddress
            else:
                harbor_ipaddress = self.harbor_server[0].ipaddress
            url = "http://{}:{}/api/v2.0/replication/policies{}".format(harbor_ipaddress,
                                                                        self.harbor_cfg["harbor"]["harbor_http_port"], policy_id)
            data = {
                "name": "replication_to_salve1",
                "dest_registry": {
                    "id": 100,
                    "name": "Harbor_slave",
                    "type": "harbor",
                    "url": "http://{}:{}".format(self.harbor_server[num_id].ipaddress,
                                                 self.harbor_cfg["harbor"]["harbor_http_port"]),
                    "credential": {
                        "type": "basic",
                        "access_key": "admin",
                        "access_secret": "{}".format(self.harbor_cfg["harbor"]["password"])
                    },
                    "insecure": False,
                },
                "dest_namespace": "",
                        "trigger": {
                            "type": model,
                            "trigger_settings": {
                                "cron": ""
                            }
                        },
                        "id": 100,
                        "enabled": True,
                        "deletion": False,
                        "override": True,
                    }
            if policy_id == '':
                res = post(url, json.dumps(data), timeout=5,
                           auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            else:
                res = put(url, json.dumps(data), timeout=5,
                          auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            if res.status_code == 200 or res.status_code == 201:
                log.info(Helper().HARBOR_REPLICATION_SUCCEED.format(res.status_code))
            else:
                log.error(Helper().HARBOR_REPLICATION_FAILED.format(res.status_code))
                exit(2)

    def get_policies_id(self):
        """
        获取复制管理的唯一ID
        :return:
        """
        url = "http://{}:{}/api/v2.0/replication/policies?name=replication_to_salve1"\
            .format(self.master_harbor.ipaddress,
                    self.harbor_cfg["harbor"]["harbor_http_port"])
        try:
            res = get(url, timeout=5,
                      auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"]))).json()[0]["id"]
            return res
        except Exception as e:
            log.error(e)

    def manual_sync(self):
        """
        触发手动同步
        :return:
        """
        data = {
            "policy_id": self.get_policies_id()
        }
        url = "http://{}:{}/api/v2.0/replication/executions"\
            .format(self.master_harbor.ipaddress,
                    self.harbor_cfg["harbor"]["harbor_http_port"])
        res = self.interface_res(url, data)
        if res.status_code != 201:
            log.error(Helper().HARBOR_SYNC_FAILED.format(res.status_code))
            exit(2)
        else:
            log.info(Helper().HARBOR_SYNC_SUCCEED.format(res.status_code))

    def _proxy_on_nginx(self):
        FileTemplate(self.harbor_cfg, self.harbor_nginx_template, self.harbor_nginx_tmp).fill()
        self.update_nginx_config()
        # self.generate_docker_compose_file(self.harbor_cfg)

    @status_me("basesystem")
    def install_harbor(self):
        self._generic_config()
        self._install_harbor()
        self._unpack_data()
        self._start()
        self._check()
        self._change_password()
        self._proxy_on_nginx()
        if len(self.harbor_server) > 1:
            self.registries()
            self.policies("manual")
            self.manual_sync()
            self.policies("event_based", '/100')





from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import status_me


class OcrIdcardController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(OcrIdcardController, self).__init__(check_result, service_path)
        self.tf_name = "ocr-idcard-server"

    @status_me("ocr_idcard")
    def push_ocr_idcard_images(self):
        self.push_images(self.project)

    @status_me("ocr_idcard")
    def start_ocr_idcard_service(self):
        self._create_namespace(namespaces=self.namespaces,
                               istio_injection_namespaces=self.istio_injection_namespaces)
        self.start_service(project=self.project, version=self.private_deploy_version, tf_name=self.tf_name)

    @status_me("ocr_idcard")
    def prepare_ocr_idcard_data(self):
        self.prepare_data(project=self.project)

    def run(self):
        self.push_ocr_idcard_images()
        self.start_ocr_idcard_service()
        self.prepare_ocr_idcard_data()



from __future__ import absolute_import
from __future__ import unicode_literals

from tqdm import tqdm


class Processor:
    def __init__(self):
        self.func_queue = {}
        self.nocols = 80
        self.ascii = ' ='
        self.bar_format = '{l_bar}{bar}|'

    def run_with_bar(self, queue_name):
        progress_bar = tqdm(iterable=self.func_queue[queue_name], ncols=self.nocols, ascii=self.ascii, bar_format=self.bar_format)
        for func in progress_bar:
            progress_bar.set_description("\t队列: {} --> 正在执行方法: {}".format(queue_name, func.__name__))
            func(self)

    def append_queue(self, queue_name, fn_name):
        try:
            self.func_queue[queue_name].append(fn_name)
        except KeyError:
            self.func_queue[queue_name] = []
            self.func_queue[queue_name].append(fn_name)

    @staticmethod
    def in_queue(queue_name):
        def get_fn(func):
            def wrap(self):
                self.append_queue(queue_name, func)
            return wrap
        return get_fn



from prettytable import PrettyTable
from laipvt.sysutil.gvalue import CHECK_FILE
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler
from laipvt.helper.errors import Helper
from laipvt.sysutil.util import log

def print_project_info():
    try:
        log.info(Helper().PRINT_PROJECT_INFO)
        tb = PrettyTable()
        tb.field_names = ["ProJect", "Desc", "URL"]
        # tb.add_row(["Adelaide", 1295, 1158259, 600.5])

        # 获取前置检查结果
        check_result_file = CHECK_FILE
        check_result = CheckResultHandler(check_result_file)
        # 获取中间件信息
        middleware_cfg = MiddlewareConfigHandler("nginx").get_all_config_with_check_result()
        project_port_map = {
            "license": {
                "desc": "License授权服务(License Service)", "port_name": ["license_web_proxy_port"]
            },
            "entuc": {
                "desc": "用户中心 (Enterprise User Center)", "port_name": ["entuc_proxy_port"]
            },
            "chatbot": {
                "desc": "ChatBot吾来对话机器人 (ChatBot)", "port_name": ["chatbot_proxy_port"]
            },
            "entcmd": {
                "desc": "机器人指挥官 (Enterprise Commander)", "port_name": ["entcmd_proxy_port"]
            },
            "mage": {
                "desc": "UiBot Mage (UiBot Mage)", "port_name": ["mage_proxy_port"]
            },
            "rpa-collaboration": {
                "desc": "人机协同中心 (RPA Collaboration)", "port_name": ["rpa_collaboration_backend_proxy_port"]
            },
            "dataservice": {
                "desc": "数据服务 (Data Service)", "port_name": ["dataservice_proxy_port"]
            },
        }
        for project in check_result.deploy_projects.get():
            if project in project_port_map:
                for port_name in project_port_map[project]["port_name"]:
                    url = "http://{}:{}/".format(middleware_cfg["nginx"]["lb"], middleware_cfg["nginx"][port_name])
                    desc = project_port_map[project]["desc"]
                    tb.add_row([project, desc, url])
        print(tb)

    except Exception as e:

        exit(2)





from __future__ import absolute_import
from __future__ import unicode_literals

import os
import time
from laipvt.handler.confighandler import CheckResultHandler, ServerHandler
from laipvt.handler.packagehandler import KubePackageHandler
from laipvt.handler.kubehandler import KubeConfigHandler
from laipvt.sysutil.util import path_join, ssh_obj, log

from laipvt.sysutil.template import FileTemplate
from laipvt.model.cmd import KubeModel


class InitInterface:
    def __init__(self, result: CheckResultHandler, kube: KubePackageHandler.parse, *args, **kwargs):
        self.result = result
        self.kube_config = KubeConfigHandler(result, *args, **kwargs).get()
        self.info = kube
        self.base_dir = path_join(result.deploy_dir, "kubernetes")

        self.kube_admin_file_template = path_join(self.info.templates, "admin.tmpl")
        self.kube_admin_file = path_join(self.info.templates, "admin.conf")

        self.hosts_file_template = path_join(self.info.templates, "hosts.tmpl")
        self.hosts_file = path_join(self.info.templates, "hosts")

        self.system_init_template = path_join(self.info.templates, "init_env.tmpl")
        self.system_init_file = path_join(self.info.templates, "init_env.sh")

        self.flannel_template = path_join(self.info.templates, "kube-flannel.tmpl")
        self.flannel_file = path_join(self.info.templates, "kube-flannel.yaml")

        self.istio_template = path_join(self.info.plugin.istio, "service/istio-private-deploy.tmpl")
        self.istio_file = path_join(self.info.plugin.istio, "service/istio-private-deploy.yaml")

        self.local_tmp = path_join(os.path.dirname(self.info.templates), "tmp")

        self.master = result.servers.get_role_obj("harbor")[0]
        self.servers = result.servers.get()

        self.network_plugin_file = path_join(
            self.info.plugin.network,
            "{}.yaml".format(self.kube_config["network_plugin"])
        )

        self.pki_tar_name = "pki.tar.gz"
        self.remote_pki_path = path_join("/etc/kubernetes", self.pki_tar_name)
        self.kubeconfig_file = "/etc/kubernetes/admin.conf"
        self.kubeconfig_local = "~/.kube/config"

    def _exec_command_to_host(self, cmd, server: ServerHandler, check_res=True):
        log.info("主机 {} 执行命令: {}".format(server.ipaddress, cmd))
        if isinstance(cmd, list):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res_list = ssh_cli.run_cmdlist(cmd)
            ssh_cli.close()
            if check_res:
                for res in res_list:
                    if res["code"] != 0:
                        log.error("{} {}".format(res["stdout"], res["stderr"]))
                        exit(2)
            return res_list
        if isinstance(cmd, str):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(cmd)
            ssh_cli.close()
            if check_res:
                if res["code"] != 0:
                    log.error("{} {}".format(res["stdout"], res["stderr"]))
                    exit(2)
            return res
        else:
            log.error("{}传入命令格式存在错误".format(cmd))
            exit(2)

    def _generate_file(self, template_file, dest_file):
        log.info("渲染{}文件到{}".format(template_file, dest_file))
        FileTemplate(self.kube_config, template_file, dest_file).fill()
        if not os.path.isfile(dest_file):
            log.error("{}文件渲染失败，程序退出".format(template_file))
            exit(2)
        return True

    def _send_file(self, src, dest, role=""):
        l = []
        if role:
            for server in self.servers:
                if server.role.check(role):
                    l.append(server)
        else:
            l = self.servers
        for server in l:
            log.info("分发{}到{}:{}".format(src, server.ipaddress, dest))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(src, dest)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _get_file(self, src, dest, server: ServerHandler):
        log.info("拉取{}:{}到{}".format(server.ipaddress, src, dest))
        if not os.path.isdir(os.path.dirname(dest)):
            os.makedirs(os.path.dirname(dest))
        ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
        try:
            ssh_cli.get(src, dest)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def add_hosts(self):
        self._generate_file(template_file=self.hosts_file_template, dest_file=self.hosts_file)
        self._send_file(self.hosts_file, "/etc/hosts")

    def install_rpms(self):
        self._send_file(self.info["k8s-rpms"], path_join(self.base_dir, "k8s-rpms"))
        cmd_list = [
            "rpm -ivUh {}/*.rpm --force --nodeps".format(path_join(self.base_dir, "k8s-rpms")),
            "unalias -a; cp -f {}/kubeadm /usr/bin/kubeadm".format(path_join(self.base_dir, "k8s-rpms"))
        ]
        for server in self.servers:
            log.info("{} 安装k8s离线包".format(server.ipaddress))
            self._exec_command_to_host(cmd=cmd_list, server=server, check_res=False)

    def system_prepare(self):
        self._generate_file(template_file=self.system_init_template, dest_file=self.system_init_file)
        self._send_file(self.system_init_file, path_join(self.base_dir, "init_env.sh"))
        cmd = "bash {}".format(path_join(self.base_dir, "init_env.sh"))
        for server in self.servers:
            log.info("{}执行初始化脚本{}".format(server.ipaddress, path_join(self.base_dir, "init_env.sh")))
            self._exec_command_to_host(cmd=cmd, server=server)

    def init_primary_master(self):
        log.info("初始化master主节点")
        self._generate_file(template_file=self.kube_admin_file_template, dest_file=self.kube_admin_file)
        self._send_file(self.kube_admin_file, path_join(self.base_dir, "admin.conf"))
        cmd = "kubeadm init --config {}".format(path_join(self.base_dir, "admin.conf"))
        self._exec_command_to_host(cmd=cmd, server=self.master)

        log.info("生成kubectl config文件")
        cmd = "mkdir ~/.kube;cp /etc/kubernetes/admin.conf ~/.kube/config"
        self._exec_command_to_host(cmd=cmd, server=self.master)

    def kube_completion(self):
        log.info("k8s命令自动补全")
        cmd = "echo 'source <(kubectl completion bash)' >> ~/.bashrc"
        self._exec_command_to_host(cmd=cmd, server=self.master)

    def install_network_plugin(self):
        log.info("部署flannel网络插件")
        self._generate_file(template_file=self.flannel_template, dest_file=self.flannel_file)
        self._send_file(self.flannel_file, path_join(self.base_dir, "kube-flannel.yaml"))
        cmd = "kubectl apply -f {}".format(path_join(self.base_dir, "kube-flannel.yaml"))
        self._exec_command_to_host(cmd=cmd, server=self.master)

    def get_kube_admin_join_command(self) -> str:
        cmd = "kubeadm token create --print-join-command --config {}".format(path_join(self.base_dir, "admin.conf"))
        res = self._exec_command_to_host(cmd=cmd, server=self.master)

        if res["code"] == 0 and "kubeadm join" in res["stdout"]:
            return res["stdout"].strip().split("\n")[-1]
        else:
            raise Exception("生成Node节点join命令失败：\n {} {}".format(res["stdout"], res["stderr"]))

    def join_master(self):
        # 1、连接master-01，打包需要拷贝的证书配置，拉到本地
        log.info("将master-01 kubernetes证书拉回本地，发送至其余master节点")
        cmd = """tar -C /etc/kubernetes -zcvf {} \
        admin.conf pki/ca.crt pki/ca.key pki/sa.key pki/sa.pub pki/front-proxy-ca.crt \
        pki/front-proxy-ca.key pki/etcd/ca.crt pki/etcd/ca.key""".format(self.remote_pki_path)
        self._exec_command_to_host(cmd=cmd, server=self.master)
        self._get_file(src=self.remote_pki_path, dest=path_join(self.local_tmp, self.pki_tar_name), server=self.master)

        other_masters = []
        for server in self.servers:
            if server.ipaddress != self.master.ipaddress:
                other_masters.append(server)

        join_cmd = self.get_kube_admin_join_command()
        for i, server in enumerate(other_masters):
            log.info("将kubernetes证书，发送至master节点{},并解压".format(server.ipaddress))
            self._send_file(src=path_join(self.local_tmp, self.pki_tar_name), dest=self.remote_pki_path)
            cmd = "tar -zxvf {} -C /etc/kubernetes/".format(self.remote_pki_path)
            self._exec_command_to_host(cmd=cmd, server=server)

            log.info("加入{}master节点".format(server.ipaddress))
            node_name = "master-0{}".format(i + 2)
            cmd = "{} --control-plane --node-name {}".format(join_cmd, node_name)
            self._exec_command_to_host(cmd=cmd, server=server)

    def join_node(self):
        for i, server in enumerate(self.result.servers.get_role_obj("node")):
            if server.ipaddress in self.result.servers.get_role_ip("master"):
                log.info("master")
                node_name = "master-0{}".format(i + 1)
                cmd = "kubectl taint nodes {} node-role.kubernetes.io/master-".format(node_name)
            else:
                cmd = self.get_kube_admin_join_command()
            self._exec_command_to_host(cmd=cmd, server=self.master)

    def check_cluster_status(self) -> bool:
        kube = KubeModel()
        status = kube.get_node_status()
        print(status)
        for i in status:
            if i["node_status"] != "Ready":
                return False
        return True

    def install_helm(self):
        self._send_file(src=path_join(self.info.plugin.helm, "bin/helm"), dest="/usr/bin/helm")
        self._send_file(src=path_join(self.info.plugin.helm, "bin/tiller"), dest="/usr/bin/tiller")
        self._send_file(
            src=path_join(self.info.plugin.helm, "service/tiller.service"),
            dest="/usr/lib/systemd/system/tiller.service"
        )
        cmd_list = [
            "chmod +x /usr/bin/helm", "chmod +x /usr/bin/tiller",
            "systemctl daemon-reload", "systemctl start tiller.service", "systemctl enable tiller.service"
        ]
        self._exec_command_to_host(cmd=cmd_list, server=self.master, check_res=True)

    def install_istio(self):
        self._send_file(src=path_join(self.info.plugin.istio, "bin/istioctl"), dest="/usr/bin/istioctl")

        self._generate_file(template_file=self.istio_template, dest_file=self.istio_file)
        istio_remote_file = path_join(self.base_dir, "istio-private-deploy.yaml")
        self._send_file(src=self.istio_file, dest=istio_remote_file)

        cmd = "chmod +x /usr/bin/istioctl && istioctl version || exit 2"
        self._exec_command_to_host(cmd=cmd, server=self.master, check_res=True)

        log.info("安装Istio")
        install_cmd = "istioctl install -f {}".format(
            path_join(self.base_dir, "istio-private-deploy.yaml")
        )
        self._exec_command_to_host(cmd=install_cmd, server=self.master, check_res=True)

        log.info("检查Istio")
        time.sleep(30)
        check_cmd = "kubectl get crds | grep 'istio.io' | wc -l"
        res = self._exec_command_to_host(cmd=check_cmd, server=self.master, check_res=True)
        try:
            if int(res["stdout"]) != 21:
                log.error("Istio CRDs 非21个，错误退出")
                exit(2)
        except Exception:
            log.error("Istio CRDs非21个，错误退出:{} {}".format(res["stdout"], res["stderr"]))
            exit(2)



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import status_me, path_join


class OcrStandardController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(OcrStandardController, self).__init__(check_result, service_path)
        self.tf_name = "ocr"


    @status_me("ocr")
    def push_ocr_images(self):
        self.push_images(self.project)

    @status_me("ocr")
    def deploy_ocr_configmap(self):
        self.deploy_all_configmap()

    @status_me("ocr")
    def start_ocr_service(self):
        self._create_namespace(namespaces=self.namespaces,
                               istio_injection_namespaces=self.istio_injection_namespaces)
        self.start_service(project=self.project, version=self.private_deploy_version, tf_name=self.tf_name)

    @status_me("ocr")
    def prepare_ocr_data(self):
        self.prepare_data(project=self.project)

    @status_me("ocr")
    def apply_ocr_model_svc(self):
        model_svc_src = path_join(self.templates_dir, "model-svc")
        model_svc_dest = path_join(self.deploy_dir, "model-svc")
        self._send_file(src=model_svc_src, dest=model_svc_dest)
        cmd = "kubectl apply -R -f {}".format(model_svc_dest)
        self._exec_command_to_host(cmd=cmd, server=self.harbor_hosts[0])

    def run(self):
        self.push_ocr_images()
        self.deploy_ocr_configmap()
        self.start_ocr_service()
        self.prepare_ocr_data()
        self.apply_ocr_model_svc()



from __future__ import absolute_import
from __future__ import unicode_literals
import os
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.handler.middlewarehandler import EtcdConfigHandler
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.helper.errors import Helper
from laipvt.sysutil.command import GET_POD_INFO, RESTART_DEPLOYMENT, RESTART_DEPLOYMENT_ALL

class LicenseController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(LicenseController, self).__init__(check_result, service_path)
        self.nginx_template = path_join(self.templates_dir, "nginx/http/nginx-license.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-license.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-license.conf")

    @status_me("license")
    def prepare_license_data_file(self, is_renew=False, **kwargs):
        # if is_renew:
        #     lcs_file_path = kwargs.get("license_file")
        # else:
        #     # 找到以.lcs结尾的文件
        #     lcs_file_path_list = list(filter(lambda x: x.endswith(".lcs"), os.listdir(self.service_path.data)))
        #     if lcs_file_path_list:
        #         lcs_file_path = path_join(self.service_path.data, lcs_file_path_list[0])
        #
        #     else:
        #         log.error(Helper().FILE_NOT_FOUND.format("license file"))
        #         exit(2)
        self.prepare_data(project=self.project)

    def restart_service(self):
        etcd = EtcdController(self.check_result, EtcdConfigHandler(), "")
        etcd.reset()
        get_pod_info_cmd = GET_POD_INFO.format("license-manager")
        res = self._exec_command_to_host(cmd=get_pod_info_cmd, server=self.harbor_hosts[0], check_res=True)
        if not (res["code"] == 0 and "Running" in res["stdout"]):
            log.error("license service not found:{}".format(get_pod_info_cmd))
            exit(2)
        restart_deployment_cmd =  RESTART_DEPLOYMENT.format("mid", "license-manager")
        self._exec_command_to_host(cmd=restart_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

        restart_all_deployment_cmd = [
            RESTART_DEPLOYMENT_ALL.format("mage"),
            RESTART_DEPLOYMENT_ALL.format("rpa")
        ]
        self._exec_command_to_host(cmd=restart_all_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

    def renew_license(self, license_file):
        self.prepare_license_data_file(is_renew=True, license_file=license_file)
        self.restart_service()

    @status_me("license")
    def push_license_images(self):
        self.push_images(self.project)

    @status_me("license")
    def start_license_service(self):
        self._create_namespace(namespaces=self.namespaces, istio_injection_namespaces="")
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("license")
    def license_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("license")
    def deploy_license_configmap(self):
        self.deploy_all_configmap()

    @status_me("license")
    def deploy_license_istio(self):
        self.deploy_istio()

    @status_me("license")
    def init_license_entuc_clients(self):
        self.init_usercenter_clients(project="license")

    def run(self):
        self.push_license_images()
        self.deploy_license_configmap()
        self.deploy_license_istio()
        self.start_license_service()
        self.prepare_license_data_file()
        self.license_proxy_on_nginx()
        self.project_pod_check()
        self.init_license_entuc_clients()




from base import check_env, registry_user, registry_hub, registry_passwd, proj_permit, check_result, prepare, proj_repo_relation
from image import ImportImage
from upgrade import CommanderUpgrade, MageUpgrade, OcrStandardUpgrade
from laipvt.sysutil.util import log, path_join
from laipvt.handler.packagehandler import ServicePackageHandler
import sys
import os

upgrade_proj = {
    "commander": CommanderUpgrade,
    "mage": MageUpgrade,
    "ocr_standard": OcrStandardUpgrade
}

def check():
    if check_env:
        log.info("环境检查通过，继续执行")
    else:
        log.error("未检测到环境信息！")
        exit(1)

def run(proj, file_path, force):
    fp = file_path.split(".")[0]
    deploy_package = prepare(file_path)
    check()
    proj_base_dir = path_join(deploy_package.root_dir, proj)
    svc_pkg = ServicePackageHandler(deploy_package.root_dir, proj)
    if not os.path.exists(proj_base_dir):
        svc_pkg.unpack()
    try:
        p = proj_repo_relation[proj]
    except:
        p = "mage"
    ImportImage(p, proj_base_dir, registry_user, registry_passwd, registry_hub)
    up = upgrade_proj[proj](fp)
    up.run(p, svc_pkg.parse(), force)

if __name__ == "__main__":
    try:
        proj = sys.argv[1]
        file_path = sys.argv[2]
        try:
            force = sys.argv[3]
            if force != True or force != False:
                exit(2)
        except IndexError:
            force = False

        if proj not in proj_permit:
            log.error("参数错误")
            exit(1)
        
        if not os.path.isfile(file_path):
            log.error("参数错误")
            exit(1)
        run(proj, file_path, force)
    except IndexError:
        log.error("参数长度错误")
        exit(1)



## gen certs command
gen_server_secure_key_cmd = "openssl genrsa -des3 -passout pass:laiye -out {} 2048"
gen_server_key_cmd = "openssl rsa -in {} -passin pass:laiye -out {}"
gen_server_csr_cmd = "openssl req -new -key {} -out {} -subj '/C=CN/ST=BeiJing/L=BeiJing/O=LaiYe Security/OU=IT Department/CN=*'"
gen_ca_crt_cmd = "openssl req -new -x509 -key {} -out {} -days 3650 -subj '/C=CN/ST=BeiJing/L=BeiJing/O=LaiYe Security/OU=IT Department/CN=*'"
gen_server_crt_cmd = "openssl x509 -req -days 3650 -in {} -CA {} -CAkey {} -CAcreateserial -out {}"

SET_UMASK = "sed -i '$aumask 0022 #laiye' /etc/bashrc"
UNSET_UMASK = "sed -i '/^umask\s0022\s#laiye/d' /etc/bashrc"

## kubenetes command
BASH_RUN = "/bin/bash {}"
CP_KUBEADM = "\cp -f {}/kubeadm /usr/bin/kubeadm"
KUBE_INIT = "kubeadm init --config {}"
CREATE_KUBE_DIR = "mkdir ~/.kube"
CP_KUBE_CONFIG = "\cp /etc/kubernetes/admin.conf ~/.kube/config"
KUBECTL_COMPLETION = "echo 'source <(kubectl completion bash)' >> ~/.bashrc"
KUBECTL_APPLY = "kubectl apply -f {}"
KUBECTL_CREATE = "kubectl create -f {}"
KUBEADM_TOKEN_CREATE = "kubeadm token create --print-join-command --config {}"
TAR_KUBE_CRETS = """tar -C /etc/kubernetes -zcvf {} \
        admin.conf pki/ca.crt pki/ca.key pki/sa.key pki/sa.pub pki/front-proxy-ca.crt \
        pki/front-proxy-ca.key pki/etcd/ca.crt pki/etcd/ca.key"""
UNTAR_KUBE_CRETS = "tar -zxvf {} -C /etc/kubernetes/"
ADD_MASTER_SUFFIX = "{} --control-plane --node-name {}"
TAINT_MASTER = "kubectl taint nodes {} node-role.kubernetes.io/master-"
HELM_PATH = "/usr/bin/helm"
TILLER_PATH = "/usr/bin/tiller"
TILLER_SERVICE_PATH = "/usr/lib/systemd/system/tiller.service"
CHMOD_EXECUTE = "chmod +x {}"
SYSTEMCTL_DAEMON_RELOAD  = "systemctl daemon-reload"
SYSTEMCTL_START = "systemctl start {}"
SYSTEMCTL_ENABLE = "systemctl enable {}"
ISTIOCTL_PATH = "/usr/bin/istioctl"
ISTIOCTL_INSTALL = "istioctl install -f {}"
CHECK_ISTIO = "kubectl get crds | grep 'istio.io' | wc -l"
GET_POD_INFO = "kubectl -n mid get pod |grep {}"
RESTART_DEPLOYMENT = "kubectl -n {} rollout restart deployment {}"
RESTART_DEPLOYMENT_ALL = "kubectl -n {} rollout restart deployment"
CREATE_NS = "kubectl create ns {}"
ISTIO_INJECTION_NS = "kubectl label namespace {} istio-injection=enabled"

CHECK_RUNNING_CMD = "kubectl -n {} get pod -l 'app={}' -o jsonpath='{{..status.phase}}'"
GET_ROOK_CEPH_DASHBOARD_PASSWORD_CMD = "kubectl -n rook-ceph get secret rook-ceph-dashboard-password -o jsonpath=\"{['data']['password']}\" | base64 --decode && echo"
GET_PVC_VOLUMENAME_CMD = "kubectl -n {} get pvc {} -o jsonpath='{{..spec.volumeName}}'"
GET_NFS_NODE_IP_CMD = "kubectl -n rook-nfs get pod -l app=rook-nfs -o jsonpath='{..status.hostIP}'"

# harbor command
TARGZ_DECOMPRESSION = "tar -zxmf {} -C {}"
CHMOD_USER_GROUP = "chown -R {}:{} {} {}"
ENABLE_HARBOR_SERVICE = "systemctl enable harbor.service"

# middleware command
RM = "rm -fr {}"
###
CREATE_DB = "create database If Not Exists `{db_name}` DEFAULT CHARSET utf8mb4 COLLATE utf8mb4_general_ci"

MKDIR_DIR = "mkdir -p {}"
CHMOD_777 = "chmod -R 777 {}"

CREATE_LICENSE_CONFIGMAP = "kubectl -n mid create configmap license-manager-config --from-literal=signature='{signature}' --from-literal=publicKey='{publicKey}'"

HELM_LIST = "helm --host=localhost:44134 list --all --chart-name {} --output=json"

# HELM_INSTALL = """helm --host=localhost:44134 install --name={process} --set global.env=pvt \
#                     --set replicaCount={replicas} \
#                     --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
#                     --set pvtWorkDir={pvt_work_dir} \
#                     --set global.LM_LOG_DEBUG=1 \
#                     --set global.LM_ETCD_ENDPOINT="{etcd_endpoint}" \
#                     --set global.LM_APPLY_LICENSE_PATH='/home/works/program/data/license.lcs' \
#                     {file_path}"""

HELM_INSTALL_CPU = """helm --host=localhost:44134 install --name={process} --set global.env=pvt \
                    --set global.namespace={namespace} \
                    --set global.component={component} \
                    --set global.logPath={logpath} \
                    --set global.processName={process_name} \
                    --set global.projectName={project_name} \
                    --set externalStorageClassName={external_storageclass_name} \
                    --set replicaCount={replicas} \
                    --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
                    --set pvtWorkDir={pvt_work_dir} \
                    --set global.LM_LOG_DEBUG=1 \
                    --set global.LM_ETCD_ENDPOINT="{etcd_endpoint}" \
                    --set global.LM_APPLY_LICENSE_PATH='/home/works/program/data/license.lcs' \
                    --set global.LM_CLIENT_SERVICE_ID="{lm_client_service_id}" \
                    --set workloadType="{workload_type}" \
                    --set resources="{resources}" \
                    {file_path}"""

HELM_INSTALL_GPU = """helm --host=localhost:44134 install --name={process} --set global.env=pvt \
                    --set global.namespace={namespace} \
                    --set global.component={component} \
                    --set global.logPath={logpath} \
                    --set global.processName={process_name} \
                    --set global.projectName={project_name} \
                    --set externalStorageClassName={external_storageclass_name} \
                    --set replicaCount={replicas} \
                    --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
                    --set pvtWorkDir={pvt_work_dir} \
                    --set global.LM_LOG_DEBUG=1 \
                    --set global.LM_ETCD_ENDPOINT="{etcd_endpoint}" \
                    --set global.LM_APPLY_LICENSE_PATH='/home/works/program/data/license.lcs' \
                    --set global.LM_CLIENT_SERVICE_ID="{lm_client_service_id}" \
                    --set workloadType="{workload_type}" \
                    --set resources.limits.nvidia\\\.com/gpu="1" \
                    --set resources.requests.nvidia\\\.com/gpu="1" \
                    {file_path}"""

INIT_IDENTITY_USER = "docker exec ids-server dotnet /opt/tool/idsadmin.dll -a add -u admin -p 123456"

HELM_INSTALL_TF_SERVICE = """helm --host=localhost:44134 install \
            --name={process} \
            --set global.env=pvt \
            --set global.namespace={namespace} \
            --set externalStorageClassName={external_storageclass_name} \
            --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
            --set pvtWorkDir={pvt_work_dir} \
            {file_path}"""

HELM_INSTALL_COMMANDER = """helm --host=localhost:44134 install --name={process} --set replicaCount={replicas} \
                    --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
                    --set pvtWorkDir={pvt_work_dir} \
                    --set config.server="{config_server}" --set config.passwd={config_server_passwd} \
                    --set mysql.host={mysql_host} --set mysql.port={mysql_port} --set mysql.user={mysql_user} \
                    --set mysql.password={mysql_password} --set mysql.database={mysql_database} --set mysql.charset={mysql_charset} \
                    --set oidc.authority={oidc_authority} --set oidc.secret={oidc_secret} \
                    --set global.LM_CLIENT_SERVICE_ID=200001 --set global.LM_CLIENT_LICENSE_MANAGER_ADDR=dns:///license-manager.mid:19080 \
                    {file_path}"""

RESTART_LICSERVER = "systemctl restart licServer"

COMMANDER_UPGRADE_INSTALL = """helm --host=localhost:44134 upgrade --install --force --set replicaCount={replicas} \
                    --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
                    --set pvtWorkDir={pvt_work_dir} \
                    --set config.server="{config_server}" --set config.passwd={config_server_passwd} \
                    --set mysql.host={mysql_host} --set mysql.port={mysql_port} --set mysql.user={mysql_user} \
                    --set mysql.password={mysql_password} --set mysql.database={mysql_database} --set mysql.charset={mysql_charset} \
                    --set oidc.authority={oidc_authority} --set oidc.secret={oidc_secret} \
                    --set global.LM_CLIENT_SERVICE_ID=200001 --set global.LM_CLIENT_LICENSE_MANAGER_ADDR=dns:///license-manager.mid:19080 \
                    {process} {file_path}"""

HELM_UPGRADE_INSTALL = """helm --host=localhost:44134 upgrade --install --set replicaCount={replicas} \
                    --set image.hub={registry_hub} --set image.name={image_name} --set image.tag={image_tag} \
                    --set pvtWorkDir={pvt_work_dir} \
                    --set global.LM_LOG_DEBUG=1 \
                    --set global.LM_ETCD_ENDPOINT="{etcd_endpoint}" \
                    --set global.LM_APPLY_LICENSE_PATH='/home/works/program/data/license.lcs' \
                    {process} {file_path}"""

# HIDE_MENU_SQL = "update laiye_saas_docuds.tbl_docuds_menu set is_deleted = 1 where menu_key in (9001, 9002, 900201, 900202, 900203);"
HIDE_MENU_SQL = "update laiye_saas_docuds.tbl_docuds_menu h join (select distinct f.menu_key, f.menu_name,f.menu_parent from laiye_saas_docuds.tbl_docuds_menu f join ( select distinct d.menu_key, d.menu_name, d.menu_parent from laiye_saas_docuds.tbl_docuds_menu d join (select distinct b.menu_key, b.menu_name, b.menu_parent from laiye_saas_docuds.tbl_docuds_menu b join (select menu_key, menu_name, menu_parent from laiye_saas_docuds.tbl_docuds_menu where menu_key in ({hide_menu_id})) a on b.menu_key = a.menu_key or b.menu_key = a.menu_parent) c on d.menu_key = c.menu_key or d.menu_key = c.menu_parent ) e on f.menu_key = e.menu_key or f.menu_key = e.menu_parent) g on h.menu_key = g.menu_key set is_deleted = 0;"
TEST_SERVICE_CMD = "docker run -ti --add-host mysql.default.svc:{} -v {}:{} -v {}:{} {}:{} --test"



import json
import re
import grpc
import requests
from google.protobuf.json_format import MessageToJson
from laipvt.sysutil.dboperation_pb2 import DbOperationRequest
from laipvt.sysutil.dboperation_pb2_grpc import DbOperationServiceStub
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR
from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.util import path_join
from laipvt.sysutil.conf import AccountIdConfig
from tenacity import retry, wait_fixed, stop_after_attempt, stop_after_delay


class Mage_test(object):
    def __init__(self, ip):
        self.cfg = YamlConfig(path_join(LAIPVT_BASE_DIR, "middleware"), suffix="yaml").read_dir()
        self.minio_port = self.cfg["minio"]["port"]
        self.mage_port = self.cfg["nginx"]["mage_proxy_port"]
        self.mage_url = "http://{}:{}".format(ip, self.mage_port)
        self.private_siber_host = "http://{}:88".format(ip)
        self.private_oss_url = "http://{}:{}/laiye-im-saas/document-mining-backend".format(ip, self.minio_port)
        self.grpc_url = "{}:31380".format(ip)
        self.account_id = AccountIdConfig().get()

    @retry(stop=stop_after_delay(300), wait=wait_fixed(2), reraise=True)
    def update_private_mage_env_info(self):
        env_info = {
            "manage_mode": "UPDATE",
            "env_info": {
                "env_name": "UiBotMage",
                "grpc": {"dev_envoy": "", "test_envoy": ""},
                "http": {
                    "dev_url": self.mage_url,
                    "test_url": self.mage_url,
                    "stage_url": self.mage_url,
                    "prod_url": self.mage_url,
                },
                "env_id": "5e940f347dcd1000016d8378",
                "insert_time": "1586761524",
                "env_mode": "interface",
                "secret_list": [],
            },
        }
        new_data = [
            {"secret_name": "通用文字识别—高速版", "app_name": "测试平台专用-文字-高速"},
            {"secret_name": "通用表格识别—高速版", "app_name": "测试平台专用-表格-高速"},
            {"secret_name": "通用多卡证识别——高速版", "app_name": "测试平台专用-智能-高速"},
            {"secret_name": "通用多票据识别——高速版", "app_name": "测试平台专用-票据-高速"},
            {"secret_name": "自定义模板识别-高速版", "app_name": "自定义模板识别-文字-高速"},
            {"secret_name": "验证码识别GENERAL", "app_name": "测试平台专用-GENERAL"},
            {"secret_name": "自定义模板识别-高精版", "app_name": "自定义模板识别-文字-高精"},
            {"secret_name": "通用文字识别—标准版", "app_name": "测试平台专用-文字-标准"},
            {"secret_name": "通用文字识别—经济版", "app_name": "测试平台专用-文字-经济"},
            {"secret_name": "通用文字识别—高精版", "app_name": "测试平台专用-文字-高精"},
            {"secret_name": "通用表格识别—标准版", "app_name": "测试平台专用-表格-标准"},
            {"secret_name": "通用表格识别—经济版", "app_name": "测试平台专用-表格-经济"},
            {"secret_name": "通用表格识别—高精版", "app_name": "测试平台专用-表格-高精"},
            {"secret_name": "文本匹配-标准版", "app_name": "测试平台专用-文本-标准"},
            {"secret_name": "验证码识别BCM", "app_name": "测试平台专用-BCM"},
            {"secret_name": "验证码识别CZB", "app_name": "测试平台专用-CZB"},
            {"secret_name": "验证码识别ICBC", "app_name": "测试平台专用-ICBC"},
            {"secret_name": "验证码识别PAB", "app_name": "测试平台专用-PAB"},
            {"secret_name": "验证码识别HZCB", "app_name": "测试平台专用-HZCB"},
            {"secret_name": "验证码识别GXRCU", "app_name": "测试平台专用-GXRCU"},
            {"secret_name": "标准地址-标准版", "app_name": "测试平台专用-地址-标准"},
            {"secret_name": "文本分类-基本版", "app_name": "测试平台专用-文本分类-基本"},
            {"secret_name": "文本分类-智能版", "app_name": "测试平台专用-文本分类-智能"},
            {"secret_name": "信息抽取—标准版", "app_name": "测试平台专用-抽取-标准"},
            {"secret_name": "语音合成-标准版", "app_name": "测试平台专用-语音合成-标准"},
            {"secret_name": "自定义模板识别-标准版", "app_name": "自定义模板识别-文字-标准"},
            {"secret_name": "自定义模板识别-高速版-表格", "app_name": "自定义模板识别-表格-高速"},
            {"secret_name": "自定义模板识别-高精版-表格", "app_name": "自定义模板识别-表格-高精"},
            {"secret_name": "通用多卡证识别—银行卡-高速版", "app_name": "测试平台专用-银行卡-高速"},
            {"secret_name": "通用多卡证识别—名片-高速版", "app_name": "测试平台专用-名片-高速"},
            {"secret_name": "通用多卡证识别—香港身份证-高速版", "app_name": "测试平台专用-香港身份证-高速"},
            {"secret_name": "通用多卡证识别—身份证-高速版", "app_name": "测试平台专用-身份证-高速"},
            {"secret_name": "通用多卡证识别—社保卡-高速版", "app_name": "测试平台专用-社保卡-高速"},
            {"secret_name": "通用多卡证识别—车辆登记证-高速版", "app_name": "测试平台专用-车辆登记证-高速"},
            {"secret_name": "通用多卡证识别—驾驶证-高速版", "app_name": "测试平台专用-驾驶证-高速"},
            {"secret_name": "通用多卡证识别—行驶证-高速版", "app_name": "测试平台专用-行驶证-高速"},
            {"secret_name": "通用多卡证识别—户口本-高速版", "app_name": "测试平台专用-户口本-高速"},
            {"secret_name": "通用多卡证识别—护照-高速版", "app_name": "测试平台专用-护照-高速"},
            {"secret_name": "通用多卡证识别—结婚证-高速版", "app_name": "测试平台专用-结婚证-高速"},
            {"secret_name": "通用多卡证识别—车辆合格证-高速版", "app_name": "测试平台专用-车辆合格证-高速"},
            {"secret_name": "通用多卡证识别—组织机构代码证-高速版", "app_name": "测试平台-组织机构代码证-高速"},
            {"secret_name": "通用多卡证识别—税务登记证-高速版", "app_name": "测试平台专用-税务登记证-高速"},
            {"secret_name": "通用多卡证识别—开户许可证-高速版", "app_name": "测试平台专用-开户许可证-高速"},
            {"secret_name": "通用多卡证识别—营业执照-高速版", "app_name": "测试平台专用-营业执照-高速"},
            {"secret_name": "通用多卡证识别—不动产证-高速版", "app_name": "测试平台专用-不动产证-高速"},
            {"secret_name": "通用多卡证识别—房产证-高速版", "app_name": "测试平台专用-房产证-高速"},
            {"secret_name": "通用多卡证识别—离婚证-高速版", "app_name": "测试平台专用-离婚证-高速"},
            {"secret_name": "自定义模板识别-经济版", "app_name": "自定义模板识别-文字-经济"},
            {"secret_name": "自定义模板识别-经济版-表格", "app_name": "自定义模板识别-表格-经济"},
            {"secret_name": "自定义模板识别-一键多值测试", "app_name": "一键多值测试"},
            {"secret_name": "自定义模板识别-预处理资源", "app_name": "自定义模板识别-预处理-高配"},
            {"secret_name": "通用多票据识别-票据-高级版CPU", "app_name": "测试平台专用-票据-CPU"},
            {"secret_name": "通用多票据识别-票据-高级版GPU", "app_name": "测试平台专用-票据-GPU"},
            {"secret_name": "通用多卡证识别-银行卡-高级版GPU", "app_name": "测试平台专用-银行卡-GPU"},
            {"secret_name": "通用多卡证识别-银行卡-高级版CPU", "app_name": "测试平台专用-银行卡-CPU"},
            {"secret_name": "通用多卡证识别-身份证-高级版CPU", "app_name": "测试平台专用-身份证-CPU"},
            {"secret_name": "通用多卡证识别-行驶证-高级版CPU", "app_name": "测试平台专用-行驶证-CPU"},
            {"secret_name": "通用多卡证识别-营业执照-高级版CPU", "app_name": "测试平台专用-营业执照-CPU"},
            {"secret_name": "通用多卡证识别-车辆登记证-高级版CPU", "app_name": "测试平台专用-车辆登记证CPU"},
            {"secret_name": "通用多卡证识别-身份证-高级版GPU", "app_name": "测试平台专用-身份证-GPU"},
            {"secret_name": "通用多卡证识别-行驶证-高级版GPU", "app_name": "测试平台专用-行驶证-GPU"},
            {"secret_name": "通用多卡证识别-营业执照-高级版GPU", "app_name": "测试平台专用-营业执照-GPU"},
            {"secret_name": "通用多卡证识别-车辆登记证-高级版GPU", "app_name": "测试平台专用-车辆登记证GPU"},
            {"secret_name": "印章识别-高级版", "app_name": "测试平台专用-印章识别-高级版"},
            {"secret_name": "印章识别-标准版", "app_name": "测试平台专用-印章识别-标准版"},
            {"secret_name": "通用文字识别-探智立方", "app_name": "测试平台专用-文字-探智立方"},
            {"secret_name": "通通用表格识别-探智立方", "app_name": "测试平台专用-表格-探智立方"},
            {"secret_name": "通用多票据识别-探智立方", "app_name": "测试平台-票据-探智立方GPU"},
            {"secret_name": "通用卡证-身份证-探智立方", "app_name": "通用卡证-身份证-探智立方"},
            {"secret_name": "自定义模板-探智立方", "app_name": "自定义模板-探智立方"},
            {"secret_name": "红头文件-高级版", "app_name": "红头文件-高级版"},
            {"secret_name": "招标公告-高级版", "app_name": "招标公告-高级版"},
            {"secret_name": "简历-高级版", "app_name": "简历-高级版"},
            {"secret_name": "简历-标准版", "app_name": "简历-标准版"},
            {"secret_name": "红头文件-标准版", "app_name": "红头文件-标准版"},
            {"secret_name": "招标公告-标准版", "app_name": "招标公告-标准版"},
            {"secret_name": "简历-高级版CPU", "app_name": "简历-高级版CPU"},
            {"secret_name": "简历-高级版GPU", "app_name": "简历-高级版GPU"},
            {"secret_name": "简历-探智立方GPU", "app_name": "简历-探智立方GPU"},
            {"secret_name": "红头文件-高级版CPU", "app_name": "红头文件-高级版CPU"},
            {"secret_name": "红头文件-高级版GPU", "app_name": "红头文件-高级版GPU"},
            {"secret_name": "红头文件-探智立方GPU", "app_name": "红头文件-探智立方GPU"},
            {"secret_name": "招标公告-高级版CPU", "app_name": "招标公告-高级版CPU"},
            {"secret_name": "招标公告-高级版GPU", "app_name": "招标公告-高级版GPU"},
            {"secret_name": "招标公告-探智立方GPU", "app_name": "招标公告-探智立方GPU"},
            {"secret_name": "合同比对-标准版", "app_name": "合同比对-标准版"},
            {"secret_name": "合同比对-高级版", "app_name": "合同比对-高级版"},
            {"secret_name": "合同比对-高级版-CPU", "app_name": "合同比对-高级版-CPU"},
            {"secret_name": "合同比对-高级版-GPU", "app_name": "合同比对-高级版-GPU"},
            {"secret_name": "合同比对-探智立方GPU", "app_name": "合同比对-探智立方GPU"},
            {"secret_name": "通用卡证-营业执照-探智立方", "app_name": "通用卡证-营业执照-探智立方"}
        ]
        # Connect to the database
        with grpc.insecure_channel(self.grpc_url) as channel:
            stub = DbOperationServiceStub(channel)
            response = stub.Operate(
                DbOperationRequest(
                    sql="SELECT name, app_key , app_secret  FROM tbl_docuds_app where is_deleted=0 and account_id = %s"
                    % self.account_id
                )
            )
            #docUnderstanding.DbOperationService
            #docUnderstanding.InnerService
            response = (
                MessageToJson(response, True, True, 2, True, False)
                .encode("ascii")
                .decode("unicode_escape")
            )
            response = json.loads(response)
            app_info_list = []
            for i in response["records"]:
                app_info = {}
                for j in range(len(response["fields"])):
                    (app_info.update({response["fields"][j]["key"]: i["values"][j]}))
                app_info_list.append(app_info)

        all_list = []
        for i in app_info_list:
            for j in new_data:
                if i["name"] == j["app_name"]:
                    secret_info = {
                        "secret_name": j["secret_name"],
                        "secret_info": {
                            "dev_secret": {
                                "pubkey": i["app_key"],
                                "secret": i["app_secret"],
                            },
                            "test_secret": {
                                "pubkey": i["app_key"],
                                "secret": i["app_secret"],
                            },
                            "stage_secret": {
                                "pubkey": i["app_key"],
                                "secret": i["app_secret"],
                            },
                            "prod_secret": {
                                "pubkey": i["app_key"],
                                "secret": i["app_secret"],
                            },
                        },
                    }
                    all_list.append(secret_info)
        env_info["env_info"].update({"secret_list": all_list})
        # print(env_info)
        response = requests.post(
            url=self.private_siber_host + "/siberhttp/manage/env", json=env_info
        )
        response_data = json.dumps(
            response.json(), sort_keys=True, indent=2, ensure_ascii=False
        )
        # print(response_data)
    # def download_oss_to_local_file():
    #     siber_host = "http://siber.wul.ai"
    #     directory = os.path.exists("../data")
    #     if not directory:
    #         os.mkdir(os.getcwd() + "/data")
    #     data = {"filter_content": {"content": "Mage", "page": "1", "page_size": "2000"}}
    #     response = requests.post(url=siber_host + "/siberhttp/list/case", json=data).json()
    #     case_info_list = response["case_info_list"]
    #     for i in case_info_list:
    #         data = {"manage_mode": "QUERY", "case_info": {"case_id": i["case_id"]}}
    #         response = requests.post(
    #             url=siber_host + "/siberhttp/manage/case", json=data
    #         ).json()
    #         for j in response["case_version"]:
    #             if "request_body" not in j.keys():
    #                 continue
    #             info = j["request_body"]
    #             if "{{FUNCTION.base_64" in info:
    #                 if download_file is True:
    #                     n = re.findall(r"{{FUNCTION.base_64\((.+?)\)}}", info)
    #                     file_name = n[0].split("/")[-1]
    #                     downloaded_obj = requests.get(n[0])
    #                     print(n[0])
    #                     with open("data/" + file_name, "wb") as file:
    #                         file.write(downloaded_obj.content)

    def replace_private_oss_url(self):
        data = {"filter_content": {"content": "Mage", "page": "1", "page_size": "2000"}}
        response = requests.post(
            url=self.private_siber_host + "/siberhttp/list/case", json=data
        ).json()
        case_info_list = response["case_info_list"]
        for i in case_info_list:
            data = {"manage_mode": "QUERY", "case_info": {"case_id": i["case_id"]}}
            response = requests.post(
                url=self.private_siber_host + "/siberhttp/manage/case", json=data
            ).json()
            # print(response)
            for j in response["case_version"]:
                if "request_body" not in j.keys():
                    continue
                info = j["request_body"]
                if "{{FUNCTION.base_64" in info:
                    n = re.findall(r"{{FUNCTION.base_64\((.+?)\)}}", info)
                    file_name = n[0].split("/")[-1]
                    new_oss_path = self.private_oss_url + "/" + file_name
                    # print(new_oss_path)
                    request_body = j["request_body"].replace(n[0], new_oss_path)
                    j["request_body"] = request_body
                    for case_version in response["case_version"]:
                        new_request = {
                            "manage_mode": "UPDATE",
                            "case_version": case_version,
                        }
                        # print("request_body:")
                        # print(new_request)
                        try:
                            response = requests.post(
                                url=self.private_siber_host + "/siberhttp/manage/case/version",
                                json=new_request,
                            )
                        except Exception as e:
                            print(e)
                        # print(response.status_code)
                        response_data = json.dumps(
                            response.json(), sort_keys=True, indent=2, ensure_ascii=False
                        )
                        # print(response_data)



from __future__ import absolute_import
from __future__ import unicode_literals
import time
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, log, status_me


class UserCenterController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(UserCenterController, self).__init__(check_result, service_path)

        self.app_config_hosts = path_join(self.config, "Usercenter/usercenter-config.conf")
        self.app_config_container = "/app/appsettings.json"

        self.nginx_template = path_join(self.templates_dir, "nginx/http/nginx-entuc.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-entuc.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-entuc.conf")

    @status_me("usercenter")
    def deploy_configmap(self):
        self.deploy_all_configmap()

    @status_me("usercenter")
    def deploy_usercenter_istio(self):
        self.deploy_istio()

    @status_me("usercenter")
    def init_usercenter_minio(self):
        self.init_minio()

    @status_me("usercenter")
    def init_usercenter_mysql(self):
        self.init_mysql()

    @status_me("usercenter")
    def init_usercenter_rabbitmq(self):
        self.init_rabbitmq()

    @status_me("usercenter")
    def push_usercenter_images(self):
        self.push_images(self.project)

    @status_me("usercenter")
    def start_usercenter_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("usercenter")
    def usercenter_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("usercenter")
    def usercenter_init_user(self):
        time.sleep(120)
        self.init_usercenter_user()

    def run(self):
        self.init_usercenter_mysql()
        self.init_usercenter_minio()
        self.init_usercenter_rabbitmq()
        self.push_usercenter_images()
        self.deploy_configmap()
        self.deploy_usercenter_istio()
        self.start_usercenter_service()
        self.usercenter_proxy_on_nginx()
        self.project_pod_check()
        self.usercenter_init_user()




from __future__ import absolute_import
from __future__ import unicode_literals

import os
from laipvt.sysutil.gvalue import DEPLOY_LANGUAGE

class Error():
    def __init__(self):
        lang = os.environ.get("DEPLOY_LANGUAGE", DEPLOY_LANGUAGE)
        map = {
            "en": self.en,
            "cn": self.cn
        }
        try:
            map[lang]()
        except KeyError:
            self.cn()

    def cn(self):
        self.UNIMPLEMENT_ERROR = "未实现的方法: {}"
        self.USER_EXIT = "用户手动退出。程序结束"

    def en(self):
        self.UNIMPLEMENT_ERROR = "un implement function: {}"
        self.USER_EXIT = "user exit. quit process"

class PackageErrors(Error):
    def __init__(self):
        super(PackageErrors, self).__init__()

    def cn(self):
        self.PROJECT_ID_UNMATCH = "授权id与实际申请的项目不匹配"
        self.PROJECT_ID_INCORRECT = "授权id不正确"
        self.PACKAGE_ILLEGAL = "文件不合法，{}不存在"
        self.UNPACK_ERROR = "文件解压失败，错误信息: {}"
        self.PACK_ERROR = "文件打包失败，错误信息: {}"

    def en(self):
        self.PROJECT_ID_UNMATCH = "authorization id is unmatch with this project"
        self.PROJECT_ID_INCORRECT = "incorrect authorization id"
        self.PACKAGE_ILLEGAL = "file type illegal，{} is not exist"
        self.UNPACK_ERROR = "file unpack failed, error msg: {}"
        self.PACK_ERROR = "package file failed, error msg: {}"

class FileTypeErrors(Error):
    def __init__(self):
        super(FileTypeErrors, self).__init__()

    def cn(self):
        self.WRONG_FILE_TYPE = "不支持的文件类型"

    def en(self):
        self.WRONG_FILE_TYPE = "unsupported file type"

class RuntimeErrors(Error):
    def __init__(self):
        super(RuntimeErrors, self).__init__()

    def cn(self):
        self.SEND_FILE_ERROR = "发送文件{}, 到服务器: {} 失败。错误信息: {}"
        self.RUN_CMD_ERROR = "在服务器: {} 执行命令 {} 失败。错误信息: {}"

    def en(self):
        self.SEND_FILE_ERROR = "Failed send {} to server: {}. Error message: {}"
        self.RUN_CMD_ERROR = "Server: {}. Run command {} failed. Error message: {}"


class Helper(Error):
    def __init__(self):
        super(Helper, self).__init__()

    def cn(self):
        self.PRECHECK_FAILED = "未执行前置检查或前置检查失败"
        self.HTTPS_CERTS_ERROR = "使用https部署，选择的是客户提供证书，证书路径不正确"
        self.FILE_NOT_FOUND = "{}文件未找到"
        self.SKIP_STEP = "{}下的{}已经执行过，跳过当前步骤"
        self.RUN_STEP = "即将运行{}下的{}方法"
        self.EXCUTE_COMMAND = "主机: {}, 执行命令: {}"
        self.COMMAND_ERROR = "命令存在错误：{}"
        self.LOCAL_FILL = "渲染{}文件到{}"
        self.FILL_ERROR = "渲染{}文件失败，程序退出: {}"
        self.SEND_FILE = "分发文件{}到{}:{}"
        self.PULL_FILE = "拉取{}:{}到{}"
        self.CHECK_STATUS = "检查{}部署状态"
        self.KUBE_MASTER_INIT = "初始化master主节点"
        self.JOIN_MASTER_NODE = "加入master节点: {}"
        self.KUBE_JOIN_COMMAND_ERROR = "生成Node节点join命令失败: {} {}"
        self.INSTALL_ISTIO = "安装Istio组件"
        self.INSTALL_ISTIO_ERROR = "安装Istio组件错误"
        self.INSTALL_ROOK = "安装rook分布式存储: {}"
        self.CHECK_CEPH_STATUS = "检查rook-ceph集群部署状态，大概需要3-5分钟，请等待..."
        self.RENEN_KUBE_APISERVER_NGINX_CONFIG = "更新kube-apiserver地址到nginx代理"
        self.DECOMPRESS_HARBOR_DATA_ERROR = "Harbor Data数据解压或授权失败:{} {}"
        self.START_MIDDLEWARE_SERVICE = "启动{} {}服务"
        self.RESTART_MIDDLEWARE_SERVICE = "重启{} {}服务"
        self.RESET_MIDDLEWARE_SERVICE = "重置{} {}服务"
        self.START_MIDDLEWARE_SERVICE_ERROR = "启动失败: {} {}"
        self.CREATE_LOG_PATH = "创建服务日志目录及用户授权: {}"
        self.CHECK_MIDDLEWARE_SERVICE = "检查主机{}服务 {}:{}"
        self.CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED = "检查主机端口{}:{}访问正常"
        self.CHECK_MIDDLEWARE_SERVICE_PORT_FAILED = "检查主机端口{}:{}访问异常"
        self.CHANGE_HARBOR_PASSWORD = "修改Harbor登陆密码"
        self.CHANGE_HARBOR_PASSWORD_SUCCEED = "修改Harbor登陆密码成功"
        self.CHANGE_HARBOR_PASSWORD_FAILED = "Harbor密码修改失败,新密码至少包含1个大写字母、1个小写字母和1个数字"
        self.HARBOR_REG_CONFIG_FAILED = "主机{} Harbor仓库管理配置失败, 状态码:{}"
        self.HARBOR_REPLICATION_SUCCEED = "Harbor复制管理配置成功"
        self.HARBOR_REPLICATION_FAILED = "Harbor复制管理配置失败"
        self.HARBOR_SYNC_SUCCEED = "Harbor 同步镜像成功, 状态码{}"
        self.HARBOR_SYNC_FAILED = "Harbor 同步镜像失败, 状态码{}"
        self.CLUSTER_REQUIRES_THREE = "集群模式最少需要三台机器"
        self.CONFIG_MYSQL_BACKUP_CRON = "配置Mysql备份定时任务"
        self.CONFIG_MYSQL_BACKUP_CRON_SUCCEED = "配置Mysql备份定时任务成功"
        self.CONFIG_MYSQL_BACKUP_CRON_FAILED = "配置Mysql备份定时任务失败"
        self.CHECK_REDIS_SERVICE_SUCCEED = "检查Redis哨兵服务通过"
        self.CHECK_REDIS_SERVICE_FAILED = "检查Redis哨兵服务失败: {}"
        self.INIT_IDENTITY_MYSQL_DATA = "初始化mysql identity数据表"
        self.CREATE_BUCKET = "创建MinIO bucket: {}"
        self.INIT_SIBER_MONGO_DATA = "初始化 siber-mongo 数据"
        self.INIT_SIBER_MONGO_DATA_ERROR = "初始化siber-mongo数据错误: {}"
        self.SIBER_CHECK_INFO = "请检查集成测试平台 '私有部署定制化mage plan' 运行结果, \n 集成测试平台访问地址http://{}:88"
        self.PUSH_IMAGE = "将镜像push到私有仓库: {}"
        self.PUSH_IMAGE_ERROR = "镜像{}导入失败：{}"
        self.TAG_IMAGE = "更新镜像tag, 旧地址: {}, 新地址: {}, tag: {}"
        self.CREATE_NAMESPACE = "创建命名空间: {}"
        self.GET_LOCAL_PATH_PVC_ERROR = "获取local path pvc错误: {}"
        self.GET_NFS_NODE_IP_ERROR = "获取NFS服务的node IP失败: {}"
        self.CREATE_PVC_ERROR = "pvc创建失败，请检查"
        self.DEPLOY_SERVICE = "服务进程开始部署: {}"
        self.SERVICE_EXISTS = "服务进程在HELM部署记录中已经存在: {}"
        self.APPTEST_SUCCEED = "{}自动化测试通过: {}"
        self.APPTEST_FAILED = "{}自动化测试未通过: {}"
        self.LOGIN_TENANT = "登录租户管理平台,请稍后..."
        self.INIT_ENTUC_USER_SUCCEED = "用户中心初始化用户成功: {}"
        self.INIT_ENTUC_USER_FAILED = "用户中心初始化用户失败: {}"
        self.GET_ENTUC_USER_JSON_ERROR = "{} 不是一个josn格式文件,请检查初始化entuc用户的返回结果"
        self.CONFIG_CLIENTS_SUCCEED = "用户中心配置client成功: {}"
        self.CONFIG_CLIENTS_FAILED = "用户中心配置client失败: {}"
        self.PRINT_PROJECT_INFO = "部署完成，以下是项目服务访问地址"
        self.PROJECT_POD_CHECK_SUCCEED = "命名空间: {}, 检查pod启动状态成功"
        self.PROJECT_POD_CHECK_FAILED = "命名空间: {}, 检查pod启动状态失败"

    def en(self):
        self.PRECHECK_FAILED = "Pre-check not excecuted or failed."
        self.HTTPS_CERTS_ERROR = "HTTPS deployments using client certificate, however, the certificate path is incorrect."
        self.FILE_NOT_FOUND = "{} File not found."
        self.SKIP_STEP = "Step {} after {} has been executed. Skip the current step."
        self.RUN_STEP = "To run {} {} method"
        self.EXCUTE_COMMAND = "Host: {}, Execute command: {}."
        self.COMMAND_ERROR = "Command Error：{}."
        self.LOCAL_FILL = "Rendering {} to {}."
        self.FILL_ERROR = "Error rendering {} file: {}"
        self.SEND_FILE = "Send file {} to {}:{}"
        self.PULL_FILE = "Pull file {}:{} to {}"
        self.CHECK_STATUS = "Check {} deploy status."
        self.KUBE_MASTER_INIT = "Initialize Kubernetes master node."
        self.JOIN_MASTER_NODE = "Join master node: {}"
        self.KUBE_JOIN_COMMAND_ERROR = "Failed to generate node join command: {} {}."
        self.INSTALL_ISTIO = "Install Istio component."
        self.INSTALL_ISTIO_ERROR = "Install Istio component ERROR."
        self.INSTALL_ROOK = "Install Rook: {}"
        self.CHECK_CEPH_STATUS = "Check rook-ceph status，it will take around 3-5 mins，please wait..."
        self.RENEN_KUBE_APISERVER_NGINX_CONFIG = "Update Kube apiserver address to nginx proxy."
        self.DECOMPRESS_HARBOR_DATA_ERROR = "Harbor data decompression or change user permisson failed: {} {}"
        self.START_MIDDLEWARE_SERVICE = "Start {} {} Service."
        self.RESTART_MIDDLEWARE_SERVICE = "Restart {} {} Service."
        self.RESET_MIDDLEWARE_SERVICE = "Reset {} {} Service."
        self.START_MIDDLEWARE_SERVICE_ERROR = "Start Failed: {} {}."
        self.CREATE_LOG_PATH = "Create a service log directory and change permission mode: {}."
        self.CHECK_MIDDLEWARE_SERVICE = "check {} service {}:{}"
        self.CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED = "Check host port {}:{} access successful."
        self.CHECK_MIDDLEWARE_SERVICE_PORT_FAILED = "Check host port {}:{} access abnomal."
        self.CHANGE_HARBOR_PASSWORD = "Change Harbor login password."
        self.CHANGE_HARBOR_PASSWORD_SUCCEED = "Change Harbor login password succeed."
        self.CHANGE_HARBOR_PASSWORD_FAILED = "Change Harbor login password failed. New password contains at least 1 uppercase letter, 1 lowercase letter and 1 number."
        self.HARBOR_REG_CONFIG_FAILED = "Host{} Harbor registry configuration failed, status code:{}"
        self.HARBOR_REPLICATION_SUCCEED = "Harbor replication configuration succeed, status code:{}"
        self.HARBOR_REPLICATION_FAILED = "Harbor replication configuration failed, status code:{}"
        self.HARBOR_SYNC_SUCCEED = "Harbor sync images succeed, status code:{}"
        self.HARBOR_SYNC_FAILED = "Harbor sync images failed,status code:{}"
        self.CLUSTER_REQUIRES_THREE = "Cluster mode requires 3 machines."
        self.CONFIG_MYSQL_BACKUP_CRON = "configure backup crontab for mysql."
        self.CONFIG_MYSQL_BACKUP_CRON_SUCCEED = "Host {} configure mysql backup crontab succeed."
        self.CONFIG_MYSQL_BACKUP_CRON_FAILED = "Host {} configure mysql backup crontab failed."
        self.CHECK_REDIS_SERVICE_SUCCEED = "Check Redis Sentinel succeed."
        self.CHECK_REDIS_SERVICE_FAILED = "Check Redis Sentinel failed.: {}"
        self.INIT_IDENTITY_MYSQL_DATA = "Initialize MySQL identity data table."
        self.CREATE_BUCKET = "Create MinIO bucket: {}"
        self.INIT_SIBER_MONGO_DATA = "Initialize siber-mongo data."
        self.INIT_SIBER_MONGO_DATA_ERROR = "Initialize siber-mongo data error: {}"
        self.SIBER_CHECK_INFO = "Please check the results of the siber platform \n Siber platform access address: http://{}:88"
        self.PUSH_IMAGE = "push image to harbor registry: {}"
        self.PUSH_IMAGE_ERROR = "Import image {} failed：{}"
        self.TAG_IMAGE = "Update image tag, old path: {}, new path: {}, tag: {}"
        self.CREATE_NAMESPACE = "Create namespace: {}"
        self.GET_LOCAL_PATH_PVC_ERROR = "Get local path pvc ERROR: {}"
        self.GET_NFS_NODE_IP_ERROR = "GET NFS node IP ERROR: {}"
        self.CREATE_PVC_ERROR = "PVC create failed."
        self.DEPLOY_SERVICE = "The service process begins deploy: {}"
        self.SERVICE_EXISTS = "The service already exists in the HELM deployment record: {}"
        self.APPTEST_SUCCEED = "{} apptest succeed: {}"
        self.APPTEST_FAILED = "{} apptest failed: {}"
        self.LOGIN_TENANT = "Logging commander tenant, please wait..."
        self.INIT_ENTUC_USER_SUCCEED = "EntUC init user succeed: {}"
        self.INIT_ENTUC_USER_FAILED = "EntUC init user failed: {}"
        self.GET_ENTUC_USER_JSON_ERROR = "{} not a json file. check usercenter init user response."
        self.CONFIG_CLIENTS_SUCCEED = "Entuc configure client succeed: {}"
        self.CONFIG_CLIENTS_FAILED = "Entuc configure client failed: {}"
        self.PRINT_PROJECT_INFO = "Deploy Succeed！The following is the access address of the project service."
        self.PROJECT_POD_CHECK_SUCCEED = "Namespace: {}, check project pod start succeed."
        self.PROJECT_POD_CHECK_FAILED = "Namespace: {}, check project pod start failed."



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import json
import re
import time
import string
import shutil
import array
import base64
import requests
import netifaces
import subprocess

from Crypto.Cipher import AES
from IPy import IP
from random import choice, shuffle, randint, sample
from laipvt.sysutil.conf import YamlConfig, JsonConfig
from laipvt.sysutil.log import Logger
from laipvt.helper.exception import UtilsError
from laipvt.sysutil.ssh import SSHConnect, monkey_sudo, to_str
from laipvt.sysutil.status import Status
from laipvt.sysutil.gvalue import LAIPVT_LOG_PATH, LAIPVT_LOG_NAME, LAIPVT_LOG_LEVEL, LOG_TO_TTY
from laipvt.helper.errors import Helper
from laipvt.sysutil.command import gen_server_key_cmd, gen_ca_crt_cmd, gen_server_crt_cmd, gen_server_csr_cmd, \
    gen_server_secure_key_cmd, SET_UMASK, UNSET_UMASK


log = Logger(
    log_path=os.environ.get("LOG_PATH", LAIPVT_LOG_PATH),
    log_name=os.environ.get("LOG_NAME", LAIPVT_LOG_NAME),
    log_level=os.environ.get("LOG_LEVEL", LAIPVT_LOG_LEVEL),
    tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY)
).get()

def ssh_obj(ip, user, password, port=22) -> SSHConnect:
    return SSHConnect(hostip=ip, username=user, password=password, port=port)

def path_join(path_1: str, *args: str):
    return os.path.join(path_1, *args)


def gen_pass(length=12, level="high") -> str:
    """
        high: 最少保证有1个特殊字符，1个大写字母，1个小写字母，1个数字。并且密码前后不可以是特殊字符。然后生成一个指定位数的密码
        middle： 包含大小写字母及数字
        low: 只生成数字密码
        :rtype: str
    """
    if level == "high":
        length = length if length > 6 else 12
#         spec = "_*+-^%{]"
        spec = "_"
        char = [string.digits, string.ascii_lowercase, string.ascii_uppercase, spec]
        # 非特殊字符开头或结尾
        match = re.compile('^[a-zA-Z0-9](?=.*\d)(?=.*[_+^%{\]\*-])(?=.*[a-z])(?=.*[A-Z]).*[a-zA-Z0-9]$')
        padding = string.ascii_letters + string.digits
    elif level == "middle":
        char = [string.digits, string.ascii_lowercase, string.ascii_uppercase]
        match = re.compile('(?=.*\d)(?=.*[a-z])(?=.*[A-Z]).*$')
        padding = string.ascii_letters + string.digits
    else:
        char = [string.digits]
        match = re.compile('(?=.*\d).*$')
        padding = string.digits
    pass_range = []
    for c in char:
        pass_range.extend(choice(c))
    pass_range.extend(choice(padding) for _ in range(length - len(pass_range)))
    shuffle(pass_range)
    password = "".join(pass_range)
    if not re.search(match, password):
        password = gen_pass(length, level)
    return password

def to_object(d: [dict, list]):
    if isinstance(d, list):
        d = [to_object(x) for x in d]
    if not isinstance(d, dict):
        return d
    class C(dict):
        __setattr__ = dict.__setitem__
        __getattr__ = dict.__getitem__
        # def get(self, key):
        #     d = self.__dict__
        #     return d[key]
        # def to_dict(self):
        #     return self.__dict__
    o = C()
    for k in d:
        o[k] = to_object(d[k])
    return o

def get_yaml_config(path):
    cfg = YamlConfig(path)
    return cfg.read_file()

def get_json_config(path):
    cfg = JsonConfig(path)
    return cfg.read_file()

def get_yaml_obj(path):
    return to_object(get_yaml_config(path))

def get_json_obj(path):
    return to_object(get_json_config(path))

def to_json(data):
    j = JsonConfig("", data=data)
    return j.read_from_data()

def file_run_able(path):
    if os.path.isfile(path):
        if os.access(path, os.X_OK):
            return True
        else:
            return False
    else:
        raise UtilsError("%s 不是合法的文件或文件不存在" % path)

def run_local_cmd(cmd, password=""):
    if password:
        cmd = monkey_sudo(password, cmd)
    code, stdout = subprocess.getstatusoutput(cmd)
    response = {
        "code": code,
        "stdout": to_str(stdout)
    }
    return to_object(response)

def local_copy(src, dest, password=""):
    cmd = "unalias -a; cp -af {} {}".format(src, dest)
    return run_local_cmd(cmd, password)

def upload(uploader, dest, src, local_path=""):
    if local_path:
        local_copy(src, local_path)
    else:
        uploader.upload_file(dest, src)

def download(path: str, url: str, file=""):
    r = requests.get(url, verify=False)
    f = os.path.join(path, file)
    if not file:
        f = os.path.join(path, url.split("/")[-1])
    with open(f, "wb") as fp:
        fp.write(r.content)
    return f

def pack(path, name, zip=False, dir_name=""):
    if not dir_name:
        dir_name = name
    cmd = "cd {} && tar cf {}.tar {}"
    p = os.path.join(path, "{}.tar".format(name))
    if zip:
        cmd = "cd {} && tar zcf {}.tar.gz {}"
        p = os.path.join(path, "{}.tar.gz".format(name))
    res = run_local_cmd(cmd.format(path, name, dir_name))
    if res.code == 0:
        return os.path.join(path, p)
    return False

def unpack(path, name, zip=False):
    cmd = "tar xmf {} -C {}"
    if zip:
        cmd = "tar zmxf {} -C {}"
    res = run_local_cmd(cmd.format(name, path))
    if res.code == 0:
        return True
    return False

def remove(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
    elif os.path.isfile(path):
        os.remove(path)
    else:
        return False
    return True

def find(path, reg, file=False):
    r = re.compile(reg)
    for root, dirs, files in os.walk(path):
        real_path = os.path.abspath(root)
        if file:
            target = files
        else:
            target = dirs
        for n in target:
            if r.match(n):
                return path_join(real_path, n)
    return False

def encode(set_array, max_len=128):
    # 向上取整
    arr_l = int((max_len + 8 - 1) / 8)
    # B代表1个字节，8bit
    arr = array.array("B", [0 for _ in range(arr_l)])
    for v in set_array:
        v = int(v)
        if v > (max_len - 1):
            raise Exception("overflow")
        arr_idx = int(v / 8)
        byte_idx = v % 8
        arr[arr_idx] = arr[arr_idx] | 1 << byte_idx
    bs = base64.encodebytes(arr.tobytes()).decode("utf8")
    # 移除掉最后的=
    return bs[0:-1]

def decode(string, max_len=128):
    # 补齐=号
    string = string + "="
    byte = base64.decodebytes(string.encode("utf8"))
    arr = array.array("B", byte)
    result = []
    for idx in range(max_len):
        arr_idx = int(idx / 8)
        byte_idx = idx % 8
        if arr[arr_idx] & (1 << byte_idx):
            result.append(idx)
    result.sort()
    return result

def get_local_net_info():
    nic = netifaces.gateways()['default'][netifaces.AF_INET][1]
    gw = netifaces.gateways()['default'][netifaces.AF_INET][0]
    for interface in netifaces.interfaces():
        if interface == nic:
            mac = netifaces.ifaddresses(interface)[netifaces.AF_LINK][0]['addr']
            try:
                ip = netifaces.ifaddresses(interface)[netifaces.AF_INET][0]['addr']
                netmask = netifaces.ifaddresses(interface)[netifaces.AF_INET][0]['netmask']
            except KeyError:
                pass
    return {
        "name": nic,
        "gw": gw,
        "mac": mac,
        "ip": ip,
        "netmask": netmask
    }

def get_net_segment(ip='', netmask=''):
    _ip = ip
    _netmask = netmask
    if not ip and not netmask:
        res = get_local_net_info()
        _ip = res['ip']
        _netmask = res['netmask']
    return IP(_ip).make_net(_netmask).strNormal()

def post(url, data=None, json=None, **kwargs):
    try:
        res = requests.post(url, data=data, json=json, **kwargs)
        return res
    except Exception as e:
        log.error(e)
        exit(1)

def put(url, data=None, **kwargs):
    try:
        res = requests.put(url, data=data, **kwargs)
        return res
    except Exception as e:
        log.error(e)
        exit(1)


def get(url, params=None, **kwargs):
    try:
        res = requests.get(url, params=None, **kwargs)
        return res
    except Exception as e:
        log.error(e)
        exit(1)

def walk_sql_path(path):
    db_info = {}
    if os.path.exists(path):
        allfilelist = os.listdir(path)
        for dir_name in allfilelist:
            filepath = os.path.join(path, dir_name)
            if os.path.isdir(filepath):
                if not dir_name.startswith("."):
                    sql_list = []
                    for i in os.listdir(filepath):
                        if i.endswith(".sql"):
                            sql_list.append(os.path.join(filepath, i))
                    db_info[dir_name] = sql_list
    # print(db_info)
    return db_info


def status_me(proj):
    def run_func(fn):
        def wrapper(self, *args, **kwargs):
            status = Status()

            if status.get_status(proj, fn.__name__) == status.STATUS_SUCCESS:
                log.info(Helper().SKIP_STEP.format(proj, fn.__name__))
                return
            else:
                status.update_status(proj, fn.__name__, status.STATUS_FAILED)
                log.info(Helper().RUN_STEP.format(proj, fn.__name__))
                fn(self, *args, **kwargs)
                status.update_status(proj, fn.__name__, status.STATUS_SUCCESS)
        return wrapper
    return run_func

def write_to_file(file_name, content=""):
    if not os.path.isdir(os.path.dirname(file_name)):
        os.makedirs(os.path.dirname(file_name))
    with open(file_name, "w", encoding="utf-8") as f:
        f.write(content)

def read_form_json_file(file_name):
    """
    读取json文件，返回dict
    """
    with open(file_name, "r") as fp:
         return json.load(fp)

def get_value_form_file(file_path, key):
    c = get_yaml_config(file_path)
    return c.get(key, False)

def backup_file(path):
    dir = os.path.dirname(path)
    if os.path.isdir(dir):
        backup_file_suffix = ".bak-%d" % int(time.time() * 1000)
        file_name = os.path.split(path)[1]
        backup_file = os.path.join(dir, file_name + backup_file_suffix)
        shutil.copyfile(path, backup_file)

def gen_https_self_signed_ca():
    https_ca_tmp_dir = "/tmp/certs"
    SERVER_SECURE_KEY = os.path.join(https_ca_tmp_dir, "server_secure.key")
    SERVER_KEY = os.path.join(https_ca_tmp_dir, "server.key")
    SERVER_CSR = os.path.join(https_ca_tmp_dir, "server.csr")
    CA_CRT = os.path.join(https_ca_tmp_dir, "ca.crt")
    SERVER_CRT = os.path.join(https_ca_tmp_dir, "server.crt")

    if not os.path.exists(https_ca_tmp_dir):
        os.mkdir(https_ca_tmp_dir)

    try:
        run_local_cmd(gen_server_secure_key_cmd.format(SERVER_SECURE_KEY))
        run_local_cmd(gen_server_key_cmd.format(SERVER_SECURE_KEY, SERVER_KEY))
        run_local_cmd(gen_server_csr_cmd.format(SERVER_KEY, SERVER_CSR))
        run_local_cmd(gen_ca_crt_cmd.format(SERVER_KEY, CA_CRT))
        run_local_cmd(gen_server_crt_cmd.format(SERVER_CSR, CA_CRT, SERVER_KEY, SERVER_CRT))
    except Exception as e:
        log.error("生成证书错误: {}".format(e))
        exit(1)

def modify_umask(check_result, is_set=True):
    cmd = SET_UMASK
    if not is_set:
        cmd = UNSET_UMASK

    for server in check_result.servers.get():
        log.info(Helper().EXCUTE_COMMAND.format(server.ipaddress, cmd))
        ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username,
                          password=server.password, port=server.port)
        res = ssh_cli.run_cmd(cmd)
        ssh_cli.close()
        if res["code"] != 0:
            log.error("{} {}".format(res["stdout"], res["stderr"]))
            exit(2)


class CommanderAes:
    """
        AES 加密解密工具
        key：加密的密钥
        text：加密的文本
        mode: 加密模式
    """

    def __init__(self):
        super(CommanderAes, self).__init__()

    @classmethod
    def add_to_16(cls, value):
        # str不是16的倍数那就补足为16的倍数
        while len(value) % 16 != 0:
            value += '\0'
        return str.encode(value)  # 返回bytes

    @classmethod
    def encrypt_oracle(cls, key, text, mode=AES.MODE_ECB):
        # 初始化加密器
        aes = AES.new(cls.add_to_16(key), mode)
        # 先进行aes加密
        encrypt_aes = aes.encrypt(cls.add_to_16(text))
        # 用base64转成字符串形式
        encrypted_text = str(base64.encodebytes(encrypt_aes), encoding='utf-8')  # 执行加密并转码返回bytes
        return encrypted_text

    @classmethod
    def decrypt_oralce(cls, key, text, mode=AES.MODE_ECB):
        aes = AES.new(cls.add_to_16(key), mode)
        # 优先逆向解密base64成bytes
        base64_decrypted = base64.decodebytes(text.encode(encoding='utf-8'))
        # 执行解密密并转码返回str
        decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')
        return decrypted_text



import os
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR, CHECK_FILE, LOG_TO_TTY
from laipvt.sysutil.ssh import SSHConnect
from laipvt.sysutil.log import Logger


class DeletePackageHandler:
    """ 删除程序功能 """

    def __init__(self):
        # 获取前置检查结果
        self.check_result_file = CHECK_FILE
        self.check_result = CheckResultHandler(self.check_result_file)
        self.deploy_dir = self.check_result.deploy_dir
        self.servers = self.check_result.servers.servers
        self.servers_all_ip = self.check_result.servers.get_all_ip()
        self.python_path = "/usr/local/python-3.7.6"
        self.laipvt_base_path = LAIPVT_BASE_DIR
        self.laipvt_middleware_path = "{}/middleware".format(self.laipvt_base_path)
        self.tmp_path = "/tmp/rpa"
        self.log = Logger(
                tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY)
        ).get()

    def stopserver(self, server: str):
        """ 停止服务 """
        cmd = "systemctl stop {server}".format(server=server)
        return cmd

    def removedir(self, dir_path: str):
        """ 移动目录到/tmp/rpa下 """
        cmd = "mv {dir_path} {tmp}".format(dir_path=dir_path, tmp=self.tmp_path)
        return cmd

    def downserver(self, yaml_path: str):
        """ Down服务 """
        cmd = "docker-compose -f {path} down".format(path=yaml_path)
        return cmd

    def checkdirpath(self, dir_path: str):
        """ 检查目录是否存在 """
        if os.path.isdir(dir_path):
            return True
        return False

    def sshclient(self, cmd):
        """ 直接执行命令 """
        for server in self.servers:
            ip = server.d["ipaddress"]
            username = server.d["username"]
            password = server.d["password"]
            port = server.d["port"]
            ssh_cli = SSHConnect(hostip=ip, username=username, password=password, port=port)
            self.log.info("{} 执行 {} 命令".format(ip, cmd))
            ssh_cli.run_cmd(cmd)
            ssh_cli.close()


class DeleteMiddleware(DeletePackageHandler):
    """ 移除各中间件服务 """
    def __init__(self):
        super(DeleteMiddleware, self).__init__()

    def get_all_middleware_name(self):
        """ 获取所有中间件的名称 """
        _path = self.laipvt_middleware_path
        if self.checkdirpath(_path):
            middleware = [os.path.splitext(fn)[0] for fn in os.listdir(_path)]
        return middleware

    def get_dockercompose_yaml(self, middleware_name):
        """ 获取docker-compose文件 """
        middleware_yaml = "{}/{}/{}".format(self.deploy_dir, middleware_name, "docker-compose.yml")
        return middleware_yaml

    def run(self):
        self.log.info("开始清除Middleware环境")
        middlewares = self.get_all_middleware_name()
        for name in middlewares:
            yaml_path = self.get_dockercompose_yaml(name)
            downcmd = self.downserver(yaml_path=yaml_path)
            mvcmd = self.removedir("{}/{}".format(self.deploy_dir, name))
            cmd = "{downcmd};{mvcmd}".format(downcmd=downcmd, mvcmd=mvcmd)
            self.sshclient(cmd)
        self.log.info("Middleware环境清理完成")


class DeleteKubernetes(DeletePackageHandler):
    """ 移除k8s """
    def __init__(self):
        super(DeleteKubernetes, self).__init__()
        self.kubernetes_path = ["/etc/kubernetes", "/var/lib/kubelet", "/var/lib/etcd", "~/.kube"]
        self.kubelet = "kubelet"
        self.tiller = "tiller"
        self.kubeadm_reset = "kubeadm reset -f"

    def run(self):
        self.log.info("开始清除Kubernetes环境")
        self.sshclient(self.kubeadm_reset)
        for path in self.kubernetes_path:
            cmd = self.removedir(path)
            self.sshclient(cmd)
        cmd = ";".join([self.stopserver(self.kubelet), self.stopserver(self.tiller)])
        self.sshclient(cmd)
        self.log.info("Kubernetes环境清理完成")


class DeleteDocker(DeletePackageHandler):
    """ 停止Docker并移除Docker """
    def __init__(self):
        super(DeleteDocker, self).__init__()
        self.docker_path = "{}/{}".format(self.deploy_dir, "Docker")

    def run(self):
        self.log.info("开始清除Docker环境")
        cmd = ";".join([self.stopserver("docker"), self.removedir(self.docker_path)])
        self.sshclient(cmd)
        self.log.info("Docker环境清理完成")


class DeleteAll(DeletePackageHandler):
    def __init__(self):
        super(DeleteAll, self).__init__()

    def Clear(self):
        """ 移除整个部署目录 """
        cmd1 = "{}".format(self.removedir(self.deploy_dir))
        self.sshclient(cmd1)
        cmd2 = "{};{}".format(self.removedir(self.laipvt_base_path), self.removedir(self.python_path))
        self.sshclient(cmd2)

    def run(self):
        """ 移除所有目录 """
        DeleteMiddleware().run()
        DeleteKubernetes().run()
        DeleteDocker().run()
        self.Clear()


def delete_main(args):
    if args.Middleware:
        """ 停止中间件服务并移除中间件目录 """
        DeleteMiddleware().run()

    if args.Docker:
        """ 移除Docker并停止Docker服务 """
        DeleteDocker().run()

    if args.Kubernetes:
        """ 移除K8S环境，并停止k8s服务 """
        DeleteKubernetes().run()

    if args.All:
        """ 移除所有环境 """
        DeleteAll().run()




from __future__ import absolute_import
from __future__ import unicode_literals

import requests
from laipvt.sysutil.util import to_json

class HarborModel():
    def __init__(self, host: str, username: str, password: str, scheme="http"):
        self.harbor = requests
        self.base_url = "%s://%s:%s@%s/api/v2.0" % (scheme, username, password, host)

    def _parse_image(self, image: str) -> dict:
        i = image.split("/")
        return {
            "project": i[0],
            "image": i[1]
        }

    def list_project(self) -> dict:
        url = self.base_url + "/projects"
        return self.harbor.get(url).json()

    def list_project_name(self) -> list:
        l = []
        data = self.list_project()
        for p in data:
            l.append(p["name"])
        return l

    def get_project_id(self, name: str) -> int:
        data = self.list_project()
        for p in data:
            if p["name"] == name:
                return p["project_id"]

    def create_project(self, name, count_limit=100, storage_limit=-1) -> bool:
        url = self.base_url + "/projects"
        data = {
            "count_limit": count_limit,
            "storage_limit": storage_limit,
            "project_name": name,
            "metadata": {
                "public": "true"
            }
        }
        self.harbor.post(url, data=to_json(data))
        if name in self.list_project_name():
            return True
        return False

    def delete_project(self, name: str) -> bool:
        id = self.get_project_id(name)
        url = self.base_url + "/projects/{}".format(id)
        self.harbor.delete(url)
        if name not in self.list_project_name():
            return True
        return False

    def list_images(self, project: str) -> dict:
        url = self.base_url + "/projects/{}/repositories".format(project)
        return self.harbor.get(url).json()

    def list_images_name(self, project: str) -> list:
        l = []
        data = self.list_images(project)
        for p in data:
            l.append(self._parse_image(p["name"])["image"])
        return l


from base import commander_config, mage_config, check_result, COMMANDER_APPSETTINGS_CONFIG, COMMANDER_APPSETTINGS
from laipvt.sysutil.util import run_local_cmd, to_object, log, path_join, write_to_file
from laipvt.model.sql import SqlModule
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler, IdentityConfigHandler, HarborConfigHandler
from laipvt.handler.packagehandler import ServicePackageHandler
from laipvt.controller.middleware.identity import IdentityController
from laipvt.controller.service.license import LicenseController
from laipvt.controller.service.mage import MageController
from laipvt.controller.service.commander import CommanderController
from laipvt.controller.service.ocr_standard import OcrStandardController
import os


class UpgradeInterface():

    def __init__(self, proj, file_path, config):
        self.proj = proj
        self.cfg = to_object(config)
        self.middleware_server_list = check_result.servers.get_role_ip("master")
        self.middleware_cfg = to_object(MiddlewareConfigHandler("mysql").get_all_config_with_check_result())
        for k, v in self.middleware_cfg.items():
            if not self.middleware_cfg[k]["ipaddress"]:
                self.middleware_cfg[k]["ipaddress"] = self.middleware_server_list
        self.file_path = file_path
        self.db_backup_dir = path_join(file_path, proj, "backup")
        self.backup_result = []
        self.db_backup = False
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        try:
            harbor_ip = self.harbor_cfg["harbor"]["ipaddress"][0]
        except IndexError:
            harbor_ip = check_result.servers.get_role_ip("harbor")[0]
        self.registry_hub = "{}:{}".format(harbor_ip, self.harbor_cfg["harbor"]["http_port"])

    
    def parse_docker_image_tag(self, image):
        return image.split(":")[-1]

    def check_version(self):
        cmd = "kubectl -n {} get deploy {} -o jsonpath={{..image}}".format(
            self.cfg.namespace, self.cfg.major_process
        )
        res = run_local_cmd(cmd)
        if res["code"] != 0:
            log.error("命令执行失败")
            log.error(res["stdout"])
            exit(2)
        else:
            return self.parse_docker_image_tag(res["stdout"])

    def check_install_identity(self):
        info = to_object({
            "name": "ids-server",
            "check_cmd": "docker ps --format \"{{ .Image }}\" --filter name=ids-server",
            "tag": "v1.1-pvt"
        })
        cmd_res = run_local_cmd(info.check_cmd)
        if cmd_res["code"] != 0:
            log.info("identity not found. deploy now")
        else:
            if cmd_res["stdout"] == info.tag:
                log.info("identity is newest version")
                return
            else:
                log.info("identity version: {}. upgrade now".format(
                    cmd_res["stdout"]
                ))
        identity_path = path_join(self.file_path, "middleware", "identity")
        identity_config = IdentityConfigHandler()
        s = IdentityController(check_result, identity_config, identity_path)
        s.deploy()

    def check_install_license(self):
        info = to_object({
            "name": "license-manager",
            "check_cmd": "kubectl -n mid get deploy license-manager -o jsonpath={..image}",
            "tag": "v2.0-pvt"
        })
        cmd_res = run_local_cmd(info.check_cmd)
        if cmd_res["code"] != 0:
            log.info("license not found. deploy now")
        else:
            if cmd_res["stdout"] == info.tag:
                log.info("license is newest version")
                return
            else:
                log.info("license version: {}. upgrade now".format(
                    cmd_res["stdout"]
                ))
        fp = ServicePackageHandler(self.file_path, "license")
        fp.unpack()
        s = LicenseController(check_result, fp.parse())
        s.deploy()


    def backup(self):
        db = SqlModule(host=self.middleware_cfg.mysql.ipaddress[0], port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        if not os.path.isdir(self.db_backup_dir):
            os.makedirs(self.db_backup_dir)
        for database in self.cfg["dbs"]:
            res = db.backup_db(database, self.db_backup_dir)
            r = {"db": database, "result": res}
            self.backup_result.append(r)
        self.db_backup = True

    def check_backup(self):
        if not self.db_backup:
            log.error("db not backup")
            exit(2)
        for i in self.backup_result:
            if i["result"]:
                lf = log.info
            else:
                lf = log.error
                lf("database {} backup failed".format(i["db"]))
            lf("db name: {}, backup status: {}".format(i["db"], i["result"]))

    def migration(self, p, force=False):
        pass

    def upgrade(self, p, service_path):
        pass
    
    def check(self):
        pass

    def run(self, p, service_path, force=False):
        self.check_install_license()
        self.check_install_identity()
        self.backup()
        self.check_backup()
        self.migration(p, force)
        self.upgrade(p, service_path)

class CommanderUpgrade(UpgradeInterface):
    def __init__(self, file_path):
        super().__init__("commander", file_path, commander_config)

    def parse_commander_tag(self, tag: str):
        # v5.5-pvt
        s = tag.split(".")
        return "{}.{}.0".format(s[0][-1], s[1].split("-")[0])

    def load_migration_image(self):
        img_path = path_join(self.file_path, self.proj, "upgrade", "commander-upgrade.tar")
        if os.path.isfile(img_path):
            cmd = "docker load < {}".format(img_path)
            run_local_cmd(cmd)
            log.info("导入数据迁移镜像成功")
        else:
            log.error("数据迁移镜像文件不存在")
            exit(3)

    def migration(self, p, force=False):
        if not force:
            if len(self.backup_result) > 0:
                for i in self.backup_result:
                    if not i["result"]:
                        log.error("database {} backup failed, cannot run migration".format(i["db"]))
                        exit(2)
        self.load_migration_image()
        if self.middleware_cfg["redis"]["is_deploy"]:
            CONFIG_SERVER = "\,".join(
                [
                    "{}:{}\,serviceName={}\,password={}\,allowAdmin=true".format(
                        server,
                        self.middleware_cfg["redis"]["port_sentinel"],
                        self.middleware_cfg["redis"]["master_name"],
                        self.middleware_cfg["redis"]["password"]
                    ) for server in check_result.servers.get_role_ip("master")
                ]
            )
        else:
            CONFIG_SERVER = "{}:{}\,password={}".format(
                self.middleware_cfg["redis"]["ipaddress"][0],
                self.middleware_cfg["redis"]["port"],
                self.middleware_cfg["redis"]["password"])
        MYSQL_ADDR = self.middleware_cfg.mysql.ipaddress[0]
        MYSQL_PORT = int(self.middleware_cfg.mysql.port)
        MYSQL_USER = self.middleware_cfg.mysql.username
        MYSQL_PASS = self.middleware_cfg.mysql.password
        db = SqlModule(host=self.middleware_cfg.mysql.ipaddress[0], port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        db.use_db("uibot_global")
        COMPANY_ID = db.select("SELECT id from tbl_company WHERE is_deleted = 0  ORDER BY id LIMIT 1")

        appsettings = COMMANDER_APPSETTINGS.format(
            CONFIG_SERVER=CONFIG_SERVER.replace('\\', ""),
            MYSQL_ADDR=MYSQL_ADDR,
            MYSQL_PORT=MYSQL_PORT,
            MYSQL_USER=MYSQL_USER,
            MYSQL_PASS=MYSQL_PASS
        )

        appsettings_config = COMMANDER_APPSETTINGS_CONFIG.format(
            CONFIG_SERVER=CONFIG_SERVER.replace('\\', ""),
            MYSQL_ADDR=MYSQL_ADDR,
            MYSQL_PORT=MYSQL_PORT,
            MYSQL_USER=MYSQL_USER,
            MYSQL_PASS=MYSQL_PASS,
            COMPANY_ID=COMPANY_ID[0]["id"]
        )
        write_to_file(path_join(self.file_path, self.proj, "upgrade", "appsettings.json"), appsettings)
        write_to_file(path_join(self.file_path, self.proj, "upgrade", "appsettings.json.config"), appsettings_config)
        old_tag = self.check_version()
        mysql_real_ip = run_local_cmd("kubectl get svc mysql -o jsonpath={..clusterIP}")["stdout"]
        cmd = self.cfg["migration"].format(
            MYSQL_REAL_IP=mysql_real_ip,
            PATH=path_join(self.file_path, self.proj, "upgrade"),
            OLD_TAG=self.parse_commander_tag(old_tag)
        )
        log.info("开始数据迁移，请勿中断操作")
        res = run_local_cmd(cmd)
        if res["code"] == 0:
            log.info("数据迁移成功")
        else:
            log.error("数据迁移失败")
            log.error(res["stdout"])
            exit(2)

    def upgrade(self, p, service_path):
        commander = CommanderController(check_result, service_path)
        commander.upgrade_service(p)
        commander.deploy_istio()

class MageUpgrade(UpgradeInterface):
    def __init__(self, file_path):
        super(MageUpgrade, self).__init__("mage", file_path, mage_config)

    def migration(self, p, force=False):
        if not force:
            if len(self.backup_result) > 0:
                for i in self.backup_result:
                    if not i["result"]:
                        log.error("database {} backup failed, cannot run migration".format(i["db"]))
                        exit(2)
        old_tag = self.check_version()
        cmd = self.cfg["migration"].format(
            REPO=path_join(self.registry_hub, p),
            NEW_TAG=self.cfg["new_version"],
            MYSQL_ADDR=self.middleware_cfg.mysql.ipaddress[0],
            MYSQL_PORT=int(self.middleware_cfg.mysql.port),
            MYSQL_USER=self.middleware_cfg.mysql.username,
            MYSQL_PASS=self.middleware_cfg.mysql.password,
            OLD_TAG=old_tag
        )
        run_local_cmd(cmd)

    def upgrade(self, p, service_path):
        mage = MageController(check_result, service_path)
        mage.deploy_configmap()
        mage.upgrade_service(p)

class OcrStandardUpgrade(UpgradeInterface):
    def __init__(self, file_path):
        super(OcrStandardUpgrade, self).__init__("ocr_standard", file_path, mage_config)

    def upgrade(self, p, service_path):
        o = OcrStandardController(check_result, service_path)
        o.upgrade_service(p)


from __future__ import absolute_import
from __future__ import unicode_literals
import os
import json
import time
from redis.sentinel import Sentinel
from laipvt.model.cmd import DockerImageModel
from laipvt.model.harbor import HarborModel
from laipvt.helper.exception import UtilsError
from laipvt.handler.confighandler import CheckResultHandler, ServerHandler, PvtAdminConfigHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, ssh_obj, to_object, walk_sql_path
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler
from laipvt.model.cmd import ComposeModel
from laipvt.model.sql import SqlModule
from laipvt.model.server import runcmd
from laipvt.helper.errors import Helper

class ServicePocInterface:
    def __init__(self, check_result: CheckResultHandler, service_path):
        """
         check_result: 对象，前置检查结果
         service_path: 对象，服务进程详情
         """
        self.check_result = check_result
        self.service_path = service_path

        self.middleware_servers = self.check_result.servers
        self.middleware_server_list = self.middleware_servers.get_role_ip("master")

        self.middleware_cfg = to_object(MiddlewareConfigHandler("mysql").get_all_config_with_check_result())
        for k, v in self.middleware_cfg.items():
            if not self.middleware_cfg[k]["ipaddress"]:
                self.middleware_cfg[k]["ipaddress"] = self.middleware_server_list
        self.middleware_cfg.update(self.check_result.__dict__)

        self.servers = check_result.servers.get()
        self.service_info = service_path.config
        self.private_deploy_version = self.service_info.tag

        self.namespace = self.service_path.config.namespace
        self.namespaces = [self.namespace, "proxy"]
        self.istio_injection_namespaces = [self.namespace, "proxy", ]
        self.project = self.namespace

        self.templates_dir = self.service_path.templates
        self.data_dir = self.service_path.data
        self.deploy_dir = self.check_result.deploy_dir

        self.service_charts_remote = path_join(self.deploy_dir, "charts")
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        try:
            harbor_ip = self.harbor_cfg["harbor"]["lb"]
        except IndexError:
            harbor_ip = self.check_result.servers.get_role_ip("harbor")[0]
        self.registry_hub = "{}:{}".format(harbor_ip, self.harbor_cfg["harbor"]["nginx_harbor_proxy_port"])

        self.etcd_servers = self.check_result.servers.get_role_ip("master")
        self.etcd_endpoint = "\,".join(
            ["{}:{}".format(server, self.middleware_cfg.etcd.http_port) for server in self.etcd_servers]
        )

        self.env_k8s_config_src = path_join(self.templates_dir, "env_k8s_config")
        self.env_k8s_config_dest = path_join(self.templates_dir, "env_k8s_config_dest")
        self.env_k8s_config_remote = path_join(self.deploy_dir, "env_k8s_config_dest")

        self.nginx_compose_file = path_join(self.deploy_dir, "nginx", "docker-compose.yml")

        self.servers = self.check_result.servers.get()
        self.middleware_cfg["k8s_hosts"] = [x.ipaddress for x in self.servers]
        self.master_hosts = self.check_result.servers.get_role_obj("master")
        self.middleware_cfg["k8s_masters"] = [x.ipaddress for x in self.master_hosts]

        self.replicas = 1
        self.nodes = self.check_result.servers.get_role_obj("node")
        self.master_host = self.check_result.servers.get_role_obj("master")[0]
        self.harbor_hosts = self.check_result.servers.get_role_obj("harbor")

        self.rabbitmq_init_file_template_path = path_join(self.templates_dir, "init_rabbitmq.tmpl")
        self.rabbitmq_init_file_path = path_join(self.templates_dir, "init_rabbitmq.sh")

        self.redis_init_file_template_path = path_join(self.templates_dir, "init_redis.tmpl")
        self.redis_init_file_path = path_join(self.templates_dir, "init_redis.sh")

        self.component = self.service_path.config.component

        # bin
        self.fill_bin_src = path_join(self.templates_dir, "pvt_gen-linux-amd64")
        self.fill_bin_remote = path_join(self.deploy_dir, "pvt_gen-linux-amd64")
        # valuePath
        self.templates_src = path_join(self.templates_dir, "env_pvt_templates")
        self.common_dest = path_join("/tmp", "{}_env_pvt_common".format(self.project))
        self.common_remote = path_join(self.deploy_dir, "{}_env_pvt_common".format(self.project))
        # tmplPath
        self.config_templates = path_join(self.templates_dir,
                                          "{}_conf_templates".format(self.component),
                                          self.component.capitalize())
        self.config_remote = path_join(self.deploy_dir,
                                       "{}_conf_templates".format(self.component),
                                       self.component.capitalize())

        self.config_target = path_join(self.deploy_dir, "{}_configmap".format(self.component))
        # config原生配置
        self.config = path_join(self.deploy_dir, "{}_config".format(self.component))

        # configmap
        self.configmap = path_join(self.config_target, self.component.capitalize())
        self.configmap_remote = path_join(self.deploy_dir, self.component.capitalize())

    def push_images(self, project):
        harbor = HarborModel(username="admin", host=self.registry_hub, password=self.middleware_cfg["harbor"]["password"])
        if project not in harbor.list_project():
            harbor.create_project(project)

        if os.path.exists(self.service_path.images):
            for image in os.listdir(self.service_path.images):
                image_path = path_join(self.service_path.images, image)
                log.info(Helper().PUSH_IMAGE.format(image_path))
                # print(image_path)
                docker = DockerImageModel(
                    image=image_path, project=project, repo=self.registry_hub, tag_version=self.private_deploy_version
                )
                docker.run()

    def proxy_on_nginx(self, nginx_template, nginx_tmp, nginx_file_remote):
        FileTemplate(self.middleware_cfg, nginx_template, nginx_tmp).fill()
        self._send_file(src=nginx_tmp, dest=nginx_file_remote)
        compose_cmd = ComposeModel(self.nginx_compose_file)
        for server in self.servers:
            self._exec_command_to_host(cmd=compose_cmd.restart(), server=server, check_res=True)

    def init_mysql(self, sql_path):
        log.info(sql_path)
        db_info = walk_sql_path(sql_path)
        if self.middleware_cfg.mysql.is_deploy:
            mysql_host = self.master_host
        else:
            mysql_host = self.middleware_cfg.mysql.ipaddress[0]
        sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        for db_name, sql_files in db_info.items():
            create_db = CREATE_DB.format(db_name=db_name)
            sql.insert_sql(create_db)
            sql.use_db(db_name)
            for sql_file in sql_files:
                sql.import_from_file(sql_file, file_eof=";\n")

    def init_rabbitmq(self):
        # log.info("渲染初始化RabbitMQ脚本: {} -- > {}".format(
        #     self.rabbitmq_init_file_template_path,
        #     self.rabbitmq_init_file_path)
        # )
        FileTemplate(self.middleware_cfg, self.rabbitmq_init_file_template_path, self.rabbitmq_init_file_path).fill()

        fp = open(self.rabbitmq_init_file_path)
        # log.info("开始执行初始化RabbitMQ命令")

        for cmd in fp.readlines():
            if cmd.strip():
                log.info(cmd)
                code, res = runcmd(cmd)
                if code != 0:
                    log.error(res)
                    exit(2)

    def init_redis(self):
        if self.middleware_cfg["redis"]["is_deploy"]:
            # 如果是自建redis服务，连接哨兵服务器，获取master节点地址
            pool = []
            for host in self.middleware_servers.get_role_obj("master"):
                redis_endpoint = (host.ipaddress, self.middleware_cfg["redis"]["port_sentinel"])
                pool.append(redis_endpoint)
            sentinel = Sentinel(pool, socket_timeout=0.5)
            # 获取主服务器地址
            master_address = sentinel.discover_master(self.middleware_cfg.redis.master_name)[0]
            # server_object_list = self.check_result.servers.search_server(key="ipaddress", value=master)
            self.middleware_cfg["redis"]["master_address"] = master_address

        # log.info("渲染初始化redis脚本: {} -- > {}".format(
        #     self.redis_init_file_template_path,
        #     self.redis_init_file_path)
        # )
        FileTemplate(self.middleware_cfg, self.redis_init_file_template_path, self.redis_init_file_path).fill()

        fp = open(self.redis_init_file_path)
        # log.info("开始执行初始化redis命令")

        for cmd in fp.readlines():
            if cmd.strip():
                log.info(cmd)
                code, res = runcmd(cmd)
                if code != 0:
                    log.error(res)
                    exit(2)

    def init_identity_user(self):
        init_user_cmd = INIT_IDENTITY_USER
        self._exec_command_to_host(cmd=init_user_cmd, server=self.servers[0], check_res=False)
